% This script gives an example of running multiple INVERSE (OPTIMIZATION) "batch_unit" batch runs.
% An example database and watershed characteristic file are used.
%
% The following batch run models are run in succession:
%   BATCH RUN  1: constant loss, match volume using FBND optimization method.
%   BATCH RUN  2: constant loss, minimize least-squares flow residuals using FBND optimization method.
%   BATCH RUN  3: proportional loss, match volume using FBND optimization method.
%   BATCH RUN  4: proportional loss, minimize least-squares flow residuals using FBND optimization method.
%   BATCH RUN  5: initial abstraction, match volume using FBND optimization method.
%   BATCH RUN  6: initial abstraction, minimize least-squares flow residuals using FBND optimization method.
%   BATCH RUN  7: initial abstraction / constant loss, minimize least-squares flow residuals using FBND
%                 optimization method.
%   BATCH RUN  8: initial abstraction / proportional loss, minimize least-squares flow residuals using
%                 FBND optimization method.
%   BATCH RUN  9: initial abstraction / constant loss, minimize least-squares flow residuals with volume
%                 match using GRID_SEARCH optimization method.
%   BATCH RUN 10: initial abstraction / proportional loss, minimize least-squares flow residuals with
%                 volume match using GRID_SEARCH optimization method.

% clean up the workspace...
close all; fclose all; diary off; clear; pack; clc;

% define the input files and directories...
databasedir = ['.',filesep,'EXAMPLE_INPUT_DATA'];         % raw data database directory to use.
WScharfile  = fullfile(databasedir,'EXAMPLE_WSchar.csv'); % watershed characteristics file to use.

%---------------------------------------------------------------------------------------------------
% BATCH RUN 1: constant loss, match volume using FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...    % run all runs found in the 'datadir' specified below.
                          ...
                  'Lfun', 'L1', ...       % L1 = use constant (Linear) loss (1-parameter) rainfall loss function.
                   'obj', 1, ...          % 1 = minimize total volume residual.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.00123], ...  % starting parameter set.
                 'min_c', [0], ...        % minimum parameter set allowed.
                 'max_c', [5], ...        % maximum parameter set allowed.
             'max_monte', 20, ...         % maximum random restarts allowed to break stationary point, if needed.
          'fmin_max_its', 1e10, ...       % maximum iterations allowed.
          'fmin_obj_tol', 1e-6, ...       % objective function tolerance stopping criteria.
                          ...
                'db_obj', 0, ...          % debug (1) or do not debug (0) the objective function (debugging gives optimization diagnostics but stops the batch).
              'save_pdf', 1, ...          % save (1) or don't save (0) output PDFs.
                          ...
            'use_synthP', 0, ...          % use synthetic (1) or use non-synthetic (actual raw) (0) rainfall.
               'P_trim1', 0.01, ...       % percent to trim the rainfall at the starting tail.
               'P_trim2', 0.01, ...       % percent to trim the rainfall at the trailing tail.
              'no_neg_P', 1, ...          % do not allow (1) or allow (0) negative rainfall after applying loss model.
           'no_neg_loss', 1, ...          % do not allow (1) or allow (0) negative rainfall loss function values (negative values result in precipitation gains).
                          ...
                'fracQp', 0.01, ...       % extend the hydrograph model until it decays to this fraction of the peak discharge.
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...  % pad the simulated and observed data with 0's to the longer of the 2.
     'max_array_size_MB', 750, ...              % maximum allowable convolution matrix size in megabytes (prevents OUT OF MEMORY errors that stop the batch).
         'USGS_DEM_AREA', 'USGS', ...           % use either the 'USGS' or 'DEM' area specified in the watershed characteristics file.
                          ...
               'datadir', databasedir, ...      % database directory to use.
              'charfile', WScharfile, ...       % watershed characteristics file to use.
       'use_last_fnames', 0);                   % use the last data files found in 'datadir' above (1) or find all data files from scratch (0).

%---------------------------------------------------------------------------------------------------
% BATCH RUN 2: constant loss, minimize least-squares flow residuals using FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'L1', ...       % L1 = use constant (Linear) loss (1-parameter) rainfall loss function.
                   'obj', 4, ...          % 4 = continuous flow least-squares.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.00123], ...
                 'min_c', [0], ...
                 'max_c', [5], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 3: proportional loss, match volume using FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'P1', ...       % P1 = use Proportional loss (1-parameter) rainfall loss function.
                   'obj', 1, ...          % 1 = minimize total volume residual.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.2], ...
                 'min_c', [0], ...
                 'max_c', [1], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 4: proportional loss, minimize least-squares flow residuals using FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'P1', ...       % P1 = use Proportional loss (1-parameter) rainfall loss function.
                   'obj', 4, ...          % 4 = continuous flow least-squares.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.2], ...
                 'min_c', [0], ...
                 'max_c', [1], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 5: initial abstraction, match volume using FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'A1', ...       % A1 = use initial Abstraction (1-parameter) rainfall loss function.
                   'obj', 1, ...          % 1 = minimize total volume residual.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.5], ...
                 'min_c', [0], ...
                 'max_c', [1], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 6: initial abstraction, minimize least-squares flow residuals using FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'A1', ...       % A1 = use initial Abstraction (1-parameter) rainfall loss function.
                   'obj', 4, ...          % 4 = continuous flow least-squares.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.5], ...
                 'min_c', [0], ...
                 'max_c', [1], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 7: initial abstraction / constant loss, minimize least-squares flow residuals using FBND
%              optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'AC2', ...      % AC2 = use initial Abstraction/Constant loss (2-parameter) rainfall loss function.
                   'obj', 4, ...          % 4 = continuous flow least-squares.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.5 0.00123], ...
                 'min_c', [0 0], ...
                 'max_c', [1 5], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);  

%--------------------------------------------------------------------------------------------------
% BATCH RUN 8: initial abstraction / proportional loss, minimize least-squares flow residuals using
%              FBND optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'AP2', ...      % AP2 = use initial Abstraction/Proportional loss (2-parameter) rainfall loss function.
                   'obj', 4, ...          % 4 = continuous flow least-squares.
              'opt_meth', 'fbnd', ...     % use fbnd constrained multivariable minimization method.
                          ...
                    'c0', [0.5 0.2], ...
                 'min_c', [0 0], ...
                 'max_c', [1 1], ...
             'max_monte', 20, ...
          'fmin_max_its', 1e10, ...
          'fmin_obj_tol', 1e-6, ...
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 9: initial abstraction / constant loss, minimize least-squares flow residuals with volume
%              match using GRID_SEARCH optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'AC2', ...           % AC2 = use initial Abstraction/Constant loss (2-parameter) rainfall loss function.
                   'obj', 4, ...               % 4 = continuous flow least-squares.
              'opt_meth', 'grid_search', ...   % use grid_search constrained multivariable minimization method.
                          ...
                    'c0', [0.5 2.5], ...
                 'min_c', [0 0], ...           % minimum parameter set allowed.
                 'max_c', [1 5], ...           % maximum parameter set allowed.
                  'n_c1', [ 20 20 50 50 100], ...       % successive grid resolution in C1 parameter direction.
                  'n_c2', [100 75 75 75 100], ...       % successive grid resolution in C2 parameter direction.
             'grid_tols', [0.5 0.25 0.1 0.05 0.01], ... % successive relative tolerances to use between sim and obs volumes.
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);

%---------------------------------------------------------------------------------------------------
% BATCH RUN 10: initial abstraction / proportional loss, minimize least-squares flow residuals with
%              volume match using GRID_SEARCH optimization method.
batch_unit( ...
            'start_stop', [1,inf], ...
                          ...
                  'Lfun', 'AP2', ...           % AP2 = use initial Abstraction/Proportional loss (2-parameter) rainfall loss function.
                   'obj', 4, ...               % 4 = continuous flow least-squares.
              'opt_meth', 'grid_search', ...   % use grid_search constrained multivariable minimization method.
                          ...
                    'c0', [0.5 0.5], ...
                 'min_c', [0 0], ...           % minimum parameter set allowed.
                 'max_c', [1 1], ...           % maximum parameter set allowed.
                  'n_c1', [ 20 20 50 50 100], ...       % successive grid resolution in C1 parameter direction.
                  'n_c2', [ 20 20 50 50 100], ...       % successive grid resolution in C2 parameter direction.
             'grid_tols', [0.5 0.25 0.1 0.05 0.01], ... % successive relative tolerances to use between sim and obs volumes.
                          ...
                'db_obj', 0, ...
              'save_pdf', 1, ...
                          ...
            'use_synthP', 0, ...
               'P_trim1', 0.01, ...
               'P_trim2', 0.01, ...
              'no_neg_P', 1, ...
           'no_neg_loss', 1, ...
                          ...
                'fracQp', 0.01, ...
 'sim_obs_trim_pad_meth', 'pad_to_longer', ...
     'max_array_size_MB', 750, ...
         'USGS_DEM_AREA', 'USGS', ...
                          ...
               'datadir', databasedir, ...
              'charfile', WScharfile, ...
       'use_last_fnames', 0);


return;


