function [] = batch_unit(varargin)
% USAGE: batch_unit('PropertyName',property_value, ...)
%
%--------------------------------------------------------------------------
% DESCRIPTION: This function performs unit hydrograph precip loss function
%   optimization in batch.
%
%--------------------------------------------------------------------------
% INPUT:
%   input parameters are passed as 'PropertyName',property_value pairs.
%   The following are supported:
%
%-------------------<< FILE AND DIRECTORY CONTROL >>-----------------------
%
%   'datadir'  - string containing the parent directory of the hydro and
%       hyetograph data files. All (paired) data files found in this directory
%       and its subdirectories will be processed.
%
%       DEFAULT: the current working directory.
%
%   'use_last_fnames' - logical determining whether to use last set if data
%       file names compiled from the previous batch run. This can save the
%       time of searching and compiling a new list from the specified
%       'datadir'. However, if the data directory structure has changed in
%       any way this will lead to file errors. The last set of data file
%       names is stored in the MAT file 'last_fnames.mat'. This file is
%       automatically created when this flag is set to 0.
%         1 = use the last set of file names, if it exists.
%         0 = pull a fresh set of data file names to use in the bacth from
%             the 'datadir' directory.
%
%   'outdir' - string indicating the directiry to place all output
%   (outfile,
%       logfile, and any pdf's).
%
%       DEFAULT: an output directory named 'OUTPUT_YYYYMMDD_HH.MM.SS_LF.OBJ'
%       will be created, where YYYY, MM, DD, HH, MM, SS are the year, month
%       day, hour, minute, and second of the batch run execution. LF and OBJ
%       are the loss and objective function specified, respectively (eg:
%       '.\OUTPUT_20060719_14.25.32_L1.1'). If this directory already exists,
%       any prior contents will be deleted.
%
%   'outfile' - string specifying the name of the tabular run results. Do
%       not include the desired directory in the name - this is specified via
%       the 'outdir' option above.
%
%       DEFAULT: 'BATCH_UNIT_OUT_YYYYMMDD_HH.MM.SS_opt_meth.Lfun.obj.csv',
%       where YYYY, MM, DD, HH, MM, SS are the year, month day, hour, minute,
%       and second of the batch run execution. "opt_meth", "Lfun", and "obj" are
%       the codes specifying the optimization method, precipitation loss function,
%       and objective functions used. For example:
%       ".\BATCH_UNIT_OUT_20060719_14.25.32_grid.AP2.4.csv"
% 
%   'logfile' - string specifying the name of the run log file. The logfile
%       saves various run details for each run in the batch. Do not include
%       the desired directory in the name - this is specified via the 'outdir'
%       option above.
%
%       DEFAULT: 'BATCH_UNIT_LOG_YYYYMMDD_HH.MM.SS_opt_meth.Lfun.obj.csv',
%       where YYYY, MM, DD, HH, MM, SS are the year, month day, hour, minute,
%       and second of the batch run execution. "opt_meth", "Lfun", and "obj" are
%       the codes specifying the optimization method, precipitation loss function,
%       and objective functions used. For example:
%       ".\BATCH_UNIT_LOG_20060719_14.25.32_grid.AP2.4.csv"
% 
%   'charfile' - string specifying the file containing the watershed
%       characteristic data for each site. If the file is not in the current
%       working directory, include the full path to the file in the string.
%       Type 'help get_ws_char' for more information regarding this file.
%
%       DEFAULT: 'ws_char.csv'. This file must exist in the current working
%       directory.
%
%--------------------<< PRECIPITATION PARAMETERS >>------------------------
%
%   'use_synthP' - logical specifying whether to use synthetic or raw
%       rainfall data. Synthetic rainfall has the same volume of water as
%       the raw rain data, but it is redistributed according to a synthetic
%       distribution. Type 'help calcP' for more information.
%         1 = use synthetic rainfall.
%         0 = use raw rainfall.
%
%       DEFAULT: 0, use raw rainfall.
%
%   'P_trim1' - number between 0 and 1 inclusive indicating the trim level
%       of the raw rain data leading edge. For example, setting to 0.01
%       will remove 1% of the total rain volume from the beginning of the
%       rain event.
%
%       DEFAULT: 0.01
%
%   'P_trim2' - number between 0 and 1 inclusive indicating the trim level
%       of the raw rain data trailing edge. For example, setting to 0.01
%       will remove 1% of the total rain volume from the end of the rain
%       event.
%
%       DEFAULT: 0.01
%
%   'no_neg_P' - logical flag specifying whether to allow negative
%       precipitation values after applying the loss function.
%         1 = do not allow negative precipitation values; any negative
%             values after applying the loss are set to 0.
%         0 = allow negative precipitation values; do not set negative
%             values to 0.
%
%       DEFAULT: 1, do not allow negative precipitation.
%
%   'no_neg_loss' - logical flag specifying whether to allow negative loss
%       function values, resulting in precipitation gains.
%         1 = do not allow negative loss function values; any negative
%             values are set to 0 prior to its application.
%         0 = allow negative loss function values; do not set negative
%             values to 0.
%
%         DEFAULT: 1, do not allow negative loss function values.
%
%---------------------<< OPTIMIZATION PARAMETERS >>------------------------
%
%   'Lfun' - string indicating which loss model to use. Type 'help
%       precip_loss' for more details on available loss functions.
%
%       DEFAULT: 'L1', 1-parameter constant loss.
%
%   'obj' - integer scalar indicating which objective function to minimize.
%       Type 'help do_run' for available 'obj' options.
%
%       DEFAULT: 1, minimize |VOLsim - VOLobs| (match total volume).
%
%   'opt_meth' - string indicating optimization method to use:
%       'fmin': uses MatLab's 'fminsearch' function for multivariate
%               function minimization. The parameters are unrestricted and
%               can attain any values. Can be used with any loss function
%               ('Lfun') and objective function ('obj'). Does not perform
%               constrained volume optimization.
%       'fbnd': uses a modified version of MatLab's 'fminsearch' function
%               ('fminsearchbnd') for multivariate function minimization. The
%               parameters are restricted to the boundaries on the parameter
%               space set by 'min_c' and 'max_c' (see below); optimal values
%               will not lie outside these boundaries. Parameters can be
%               selectively bounded by setting individual upper, lower, or
%               both bounds to -inf or inf. An individual parameter can be
%               held fixed by setting its upper and lower bound to the same
%               value. Can be used with any loss function ('Lfun') and
%               objective function ('obj'). Does not perform constrained
%               volume optimization.
%       'grid_search': uses an adaptive parameter space grid search
%               scheme where canidate parameter pairs are selected that
%               meet sim/obs volume match and 'zooms' in on the grid to
%               obtain a final set of canidates. The objective function
%               'obj' is then evaluated for the final canidate list, and
%               the parameters resulting in the smallest objective function
%               value is selected as the optimal parameters. Run parameters
%               'n_c1', 'n_c2', and 'grid_tols' should be specified to direct
%               the 'zooming' procedure. ONLY USABLE WITH 2-PARAMETER LOSS
%               FUNCTIONS.
%
%       DEFAULT: 'fbnd'
%
%   'FMIN' /'FBND' PARAMETERS...
%
%   'c0' - either:
%       (1) numeric scalar or vector containing the starting values of the
%           loss function parameters. A 1-parameter loss function requires a
%           scalar, a 2-parameter loss function requires a 2-element vector
%           [c1,c2], etc. The values used have dramatic impact on optimization
%           convergence.
%       (2) character string containing a csv file name of c0 values to use for
%           each station. The first column is the station ID, and remaining
%           columns are the values to use as c0 for the station as described above.
%           A single column is a scalar c0 value, and multiple columns is the
%           vector c0 value. This option is usually used when running a batch
%           without optimization (see 'opt_meth' above), where each station has
%           its own c0 value to use.
%
%       DEFAULT: 0.000123 (intended for use with a 1-parameter loss function).
%
%   'min_c' - a real scalar or vector with the same size as c0 used in
%       bounded minimizations (see 'opt_meth' above) and attempting to
%       break a stationary point (see 'max_monte' below).
%
%       DEFAULT: 0 (intended for use with a 1-parameter loss function).
%
%   'max_c' - a real scalar or vector with the same size as c0 used in
%       bounded minimizations (see 'opt_meth' above) and attempting to
%       break a stationary point (see 'max_monte' below).
%
%       DEFAULT: 5 (intended for use with a 1-parameter loss function).
%
%   'fmin_max_its' - positive interger specifying the maximum iterations
%       allowed by the 'fminsearch' algorithm. Generally, each iteration
%       requires 2 objective function evaluations.
%
%       DEFAULT: 1e10 (essentially unrestricted iterations)
%
%   'fmin_obj_tol' - positive real number specifying the stopping tolerance
%       used by the 'fminsearch' algorithm. The algorithm stops when:
%                 | obj(i) - obj(i-1) | <= fmin_obj_tol
%
%       DEFAULT: 1e-6
%
%   'max_monte' - positive integer specifying the maximum number of
%        randomization iterations allowed for breaking out of optimization
%        stationary points. If the optimizer returns an 'optimized' result
%        that is identical to the initial input guess c0 (defined above),
%        this is considered a stationary point (ie: a zero-gradient
%        objective function point). In this case, the optimizer will be
%        restarted at a randomly selected c0 value within the specified
%        'min_c' and 'max_c' values (see below). If this procedure is
%        unsuccessful after 'max_monte' tries, the run is skipped.
%
%        DEFAULT: 20
%
%   'GRID_SEARCH' PARAMETERS...
%   NOTE: the following must have the same number of elements.
%
%   'n_c1' - integer vector containing successive grid resolutions in the c1
%        direction  to use for each grid search pass. The length of the vector
%        determines the number of search passes. In the ith pass, the
%        parameter space is divided into n_c1(i) parts in the c1 direction.
%
%        DEFAULT: [20  20   50  50   100] (5 passes with increasing resolution)
%
%   'n_c2' - integer vector containing successive grid resolutions in the c2
%        direction  to use for each grid search pass. The length of the vector
%        determines the number of search passes. In the ith pass, the
%        parameter space is divided into n_c2(i) parts in the c2 direction.
%
%        DEFAULT: [20  20   50  50   100] (5 passes with increasing resolution)
%
%   'grid_tols' - numerical vector containing the relative tolerances to use
%        between sim and obs volumes for each pass. During the ith pass,
%        those loss function parameters that meet sim/obs volume match
%        within 'grid_tols(i)' will be selected as canidate parameters for final
%        optimization. On the last pass, the canidate list is reduced to
%        those meeting 'grid_tols(end)', and the parameters from this list with
%        the smallest objective function evaluation (specified by 'obj')
%        will be selected as the optimized parameter pair.
%
%        DEFAULT: [0.5 0.25 0.1 0.05 0.01] (5 passes with tightening tolerances)
%
%   'min_c' - a real scalar or vector with the same size as c0 used for the
%        starting lower bounds of the parameter space. Successive grid search
%        passes will 'zoom in' this starting space to converge on an
%        optimal set of canidate parameter pairs that satisfy the specified
%        final volume match tolerance.
%
%        DEFAULT: 0 (intended for use with a 1-parameter loss function).
%
%   'max_c' - a real scalar or vector with the same size as c0 used for the
%        starting upper bounds of the parameter space. Successive grid search
%        passes will 'zoom in' this starting space to converge on an
%        optimal set of canidate parameter pairs that satisfy the specified
%        final volume match tolerance.
%
%        DEFAULT: 5 (intended for use with a 1-parameter loss function).
%
%--------------------------<< MISCELLANEOUS >>-----------------------------
%
%   'save_pdf' - logical: 1 = save diagnostic plots of each run as a PDF file.
%                         0 = do not save any diagnostic plots.
%
%       DEFAULT: 1, plots will be created.
%
%   'start_stop' - integer vector in form [start,stop] designating the range
%       of runs to perform. The batch will start with run # start in the input
%       file list, and end with run # stop. This allows prematurely stopped
%       runs to resume where they left off, and also allows a batch run to be
%       spread across multiple machines, provided they are using the same input
%       data directory structures. Set 'stop' to inf to run the batch from
%       'start' to completion.
%
%       DEFAULT: [1,inf], all runs in the batch will be performed.
%
%   'USGS_DEM_AREA' - string, either 'USGS' or 'DEM' specifying which
%       watershed area values to use from the waterseh characteristics file
%       (see 'charfile' above). USGS values are from site records, whereas
%       DEM values were obtained from a digital elevation map.
%
%       DEFAULT: 'USGS'
% 
%   'max_array_size_MB' - positive real number indicating the maximum
%       allowable convolution matrix size in megabytes. This is useful
%       for avoiding 'out of memory' errors during a batch run that can
%       occur for excessively long storm events. These errors otherwise
%       stop the batch. The optimal value depends on the machine being
%       used.
%
%       DEFAULT: 750
% 
%   'fracQp' - real scalar between 0 and 1 specifying how long in time to run
%       model. Model terminates when the gamma unit hydrograph decays to
%       'fracQp' of the theoretical peak discharge of the model gamma unit
%       hydrograph.
%
%       DEFAULT: 0.05
%
%   'sim_obs_trim_pad_meth' - string indicating the trim/pad method to use
%       on the simulated (model) and observed (raw) time-series. The final
%       trimmed/padded values are used for all output statistics and plots.
%       Can be one of 4 options:
%         (1) 'trim_pad_to_obs': observed discharge and volume time-series
%             are not altered; simulated discharge and volume time-series are
%             either trimmed or padded to the same length as the observed
%             time-series.
%         (2) 'trim_pad_to_sim': simulated discharge and volume time-series
%             are not altered; observed discharge and volume time-series are
%             either trimmed or padded to the same length as the simulated
%             time-series.
%         (3) 'trim_to_shorter': If the simulated time-series are longer
%             than the observed time-series, they are trimmed to the same
%             length of the observed time-series; if the observed time-series
%             are longer than the simulated time-series, they are trimmed
%             to the same length of the simulated time-series.
%         (4) 'pad_to_longer': If the simulated time-series are shorter
%             than the observed time-series, they are padded to the same
%             length of the observed time-series; if the observed time-series
%             are shorted than the simulated time-series, they are padded
%             to the same length of the simulated time-series.
%
%       DEFAULT: 'pad_to_longer'
%
%   'db_obj' - debugging diagnostic, logical:
%                1 = debug mode: run as normal, but create plot of objective
%                    function space and stop batch.
%                0 = regular run mode: do not create objective fuction space plot.
%              NOTES:
%                * an objective function plot can only be created for 1- or
%                  2-parameter loss functions.
%                * if 'opt_meth' is set to 'grid_search', results of the
%                grid search for each successive pass will also be generated.
%
%       DEFAULT: 0, regular run mode.
%
%--------------------------------------------------------------------------
% OUTPUT:
%   There are 3 types of output, all of which are placed in the output
%   directory specified by the 'outdir' parameter:
%
%   (1) A comma delimited output text file containing the results of the run.
%       It is named as specified by the 'outfile' parameter.
%
%   (2) A log file capturing all command line output during the run. It is
%       named  as specified by the 'outfile' parameter.
%
%   (3) An individual PDF file (optional depending on the 'save_pdf' setting)
%       will also be generated for each run in the batch. They are named
%       'run####_sta########_YYYY_MMDD.pdf', where #### is the run number,
%       ######## is the 8-digit station ID, and YYYY, MM, DD are the year,
%       month, and day of the hydrologic event, respectively
%       (eg: run0021_sta08158880_1978_0212.pdf). These files can later be
%       combined into a single PDF binder file.
%
%--------------------------------------------------------------------------
% Written by Joe Vrabel, USGS May2006

usage = 'USAGE: batch_unit(''PropertyName'',property_value, ...)';

% start timer...
tic;

% close all figures, open files, and log file...
close all;
fclose all;
diary off;

% parse input args and assign values to Hin data structure...
[Hin] = parse_input(varargin);
% disp(Hin);
% error('DEBUG STOP');

% create outdir if needed - if exists, delete contents...
if exist(Hin.outdir,'dir') == 7, % already exists - erase it and its contents.
    rmdir(Hin.outdir,'s');
end
mkdir(Hin.outdir); % make fresh (empty) plot directory.

% start diary log...
if exist(Hin.logfile,'file') == 2,
    delete(Hin.logfile);
end
diary(Hin.logfile);

% echo run info...
echo_run_info(varargin,Hin);
disp('--------------------------------------------------------------');

%--------------------------------------------------------------------------
% LOAD C0 VALUES TO USE IF C0 ENTERED AS A STRING
if ischar(Hin.c0),
    disp('Loading c0 loss function parameters to use...');
    staid_c0 = get_c0(Hin.c0);
    n_c0 = size(staid_c0,2) - 1; % number of loss parameters.
    c0file = 1; % flag that file used.
else % numeric c0.
    n_c0 = numel(Hin.c0); % number of loss parameters.
    c0file = 0; % flag that file used.
end

%--------------------------------------------------------------------------
% CREATE OUTPUT FILE

% create output file...
header = [ ... % NOTE - goto write_out.m for updating header text.
    'RUN_NUM,' ...
    'DATADIR,' ...
    'STAID,' ...
    'YYYYMMDD,' ...
    'AREA[mi^2],' ...
    'CHAN_LENGTH[mi],' ...
    'SLOPE,' ...
    'DEVELOPED,' ...
    'GAMMA_Tp[hr],' ...
    'GAMMA_Qp[CFS],' ...
    'GAMMA_Qp[in/hr],' ...
    'GAMMA_K,' ...
    'N_MOD_PEAKS,' ...
    'N_POINTS,' ...
    'BEGIN_RAIN,' ...
    'RAIN_TOT_VOL[in],' ...
    'RAIN_XS_VOL[in],' ...
    'RAIN_PER_VOL_LOSS,' ...
    'TpOBS_TpSIM_DIFF[hrs],' ...
    'OBS_Q_MEAN[CFS],' ...
    'SIM_Q_MEAN[CFS],' ...
    'OBS_Q_MAX[CFS],' ...
    'SIM_Q_MAX[CFS],' ...
    'RMS_Q_RESID[CFS],' ...
    'Q_ERR_VAR[CFS^2],' ...
    'Q_REL_BIAS,' ...
    'Q_REL_STD_ERR,' ...
    'Q_NS_EFF,' ...
    'Q_SIM_OBS_R2,' ...
    'Q_SIM_OBS_SLOPE,' ...
    'Q_SIM_OBS_INT,' ...
    'OBS_V_MEAN[CFS],' ...
    'SIM_V_MEAN[CFS],' ...
    'OBS_V_MAX[CFS],' ...
    'SIM_V_MAX[CFS],' ...
    'OBS_V_TOT[in],' ...
    'SIM_V_TOT[in],' ...
    'RMS_V_RESID[CFS],' ...
    'V_ERR_VAR[CFS^2],' ...
    'V_REL_BIAS,' ...
    'V_REL_STD_ERR,' ...
    'V_NS_EFF,' ...
    'V_SIM_OBS_R2,' ...
    'V_SIM_OBS_SLOPE,' ...
    'V_SIM_OBS_INT,' ...
    'SIM_OBS_TOT_VOL_RATIO,' ...
    'LOSS_MODEL,' ...
    'OBJ_FUNC,' ...
    'OBJ_OPT,' ...
    ];
% add as many C_OPT columns as needed to header...
for i = 1:(n_c0-1),
    header = [header,['C_OPT(',num2str(i),'),']]; % add another one.
end
header = [header,['C_OPT(',num2str(n_c0),')']]; % add last one.
try
    fid = fopen(Hin.outfile,'w'); % create fresh outfile.
    fprintf(fid,'%s\n',header);
    fclose(fid);
catch
    error(['Cannot open output file: ',Hin.outfile]);
end

%--------------------------------------------------------------------------
% LOAD WATERSHED CHARACTERISTICS
disp('Loading watershed characteristics...');
ws_char = get_ws_char(Hin.charfile,Hin.USGS_DEM_AREA); % ws_char = [STAIDS,AREAS,LENGTHS,SLOPES,DEVELS]

%--------------------------------------------------------------------------
% GET DATA FILES
disp('Getting data files...');
if ~Hin.use_last_fnames, % get all available data file names from scratch.
    [hydro_files,hyeto_files] = get_files(Hin.datadir,ws_char);
else % load last saved set of data file names.
    if exist('last_fnames.mat','file') == 2, % file exists - load.
        load_struct = load('last_fnames.mat');
        hydro_files = load_struct.hydro_files;
        hyeto_files = load_struct.hyeto_files;
        disp('Last used data file names loaded from ''last_fnames.mat''.');
    else
        disp(usage);
        error('Cannot load last set of data file names - file ''last_fnames.mat'' does not exist.');
    end
end
if isempty(hydro_files), % no files found!
    disp('<< No data files found >>');
    error('STOPPING.');
end
n_all_run = length(hydro_files);
% adjust start_stop as needed...
if isinf(Hin.start_stop(2)),
    Hin.start_stop(2) = n_all_run;
end
if Hin.start_stop(1) > n_all_run,
    error(['Specified starting run (',num2str(Hin.start_stop(1)), ...
    ') is greater than the number of runs in the batch (',num2str(n_all_run),')']);
end
if Hin.start_stop(2) > n_all_run,
    disp(['NOTICE: Specified stopping run (',num2str(Hin.start_stop(1)), ...
    ') is greater than the number of runs in the batch (',num2str(n_all_run), ...
    ') - batch will be run until completion.']);
    Hin.start_stop(2) = n_all_run;
end

%--------------------------------------------------------------------------
% PERFORM BATCH
disp('Performing batch...');
disp(' ');
n_run = Hin.start_stop(2) - Hin.start_stop(1) + 1; % number of runs in specified batch.
run_count = 0; % incremental run count.
for run = Hin.start_stop(1):Hin.start_stop(2),
    run_count = run_count + 1;
    
    % store parms in Hin structure...
    Hin.hydro_file = hydro_files{run};
    Hin.run_num = run;
    
    disp('==============================================================');
    disp(['RUN #',num2str(run),' (',num2str(run_count),' OUT OF ',num2str(n_run),')']);
    disp(['    HYDROGRAPH DATA FILE: ',hydro_files{run}]);
    disp(['    HYETOGRAPH DATA FILE: ',hyeto_files{run}]);
    staid1 = get_staid(hydro_files{run}); % get staid from hydro filename.
    if isempty(staid1) || isnan(staid1), % non-number staid.
        disp(['<< UNRECOGNIZED STAID FOR: ',hydro_files{run},' - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end
    staid2 = get_staid(hyeto_files{run}); % get staid from hyeto filename.
    if isempty(staid2) || isnan(staid2), % non-number staid.
        disp(['<< UNRECOGNIZED STAID FOR: ',hyeto_files{run},' - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end
    if staid1 ~= staid2,
        error(['<< Non-matching hydro-hyeto staid pair encountered: ',zeropad(staid1,8),',',zeropad(staid2,8),' >>']);
    else
        staid = staid1;
    end
    
    % get watershed characteristics...
    [Hin.AREA,Hin.LENGTH,Hin.SLOPE,Hin.DEVEL] = lookup_ws_char(ws_char,staid);
    if isempty(Hin.AREA) || isempty(Hin.LENGTH) || isempty(Hin.SLOPE) || isempty(Hin.DEVEL), % not in list.
        disp(['<< WATERSHED CHARACTERISTICS NOT FOUND FOR STATION: ',zeropad(staid,8),' - SKIPPING >>']);
        Hin.AREA = '';
        Hin.LENGTH = '';
        Hin.SLOPE = '';
        Hin.DEVEL = '';
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end
    disp( '    WATERSHED CHARACTERISTICS:');
    disp(['      * STAID  = ',zeropad(staid,8)]);
    disp(['      * AREA   = ',num2str(Hin.AREA),  ' [mi^2]']);
    disp(['      * LENGTH = ',num2str(Hin.LENGTH),' [mi]']);
    disp(['      * SLOPE  = ',num2str(Hin.SLOPE), ' [mi/mi]']);
    disp(['      * DEVEL  = ',num2str(Hin.DEVEL), ' [0=undeveloped, 1=developed]']);
    
    % get c0 to use if file method used...
    if c0file,
        Hin.c0 = lookup_c0(staid_c0,staid);
        if isempty(Hin.c0),
            disp(['<< C0 NOT FOUND FOR STATION: ',zeropad(staid,8),' - SKIPPING >>']);
            [Hin] = write_out(Hin,'null'); % write null results to output file.
            continue; % go on to next data file.
        end
        disp(['    C0 TO USE LOADED FROM FILE: ',num2str(Hin.c0)]);
    end

    % read unit file...
    disp('    Reading unit data...');
    [Hin.Q_RAW,err] = read_data(hydro_files{run});
    if err,
        disp(['<< Cannot read unit file: ',hydro_files{run},' - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end
    if length(Hin.Q_RAW) < 3,
        disp(['<< Not enough data points (',num2str(length(Hin.Q_RAW)),') in unit file: ',hydro_files{run},' - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end
    
    % read rain file...
    disp('    Reading rain data...');
    [Hin.P_RAW,err] = read_data(hyeto_files{run});
    if err,
        disp(['<< Cannot read rain file: ',hyeto_files{run},' - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end
    if length(Hin.P_RAW) < 3,
        disp(['<< Not enough data points (',num2str(length(Hin.P_RAW)),') in rain file: ',hyeto_files{run},' - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end

    % check that tot volume of rain >= total runoff...
    if Hin.P_RAW(end,2) < (Hin.Q_RAW(end,2)-Hin.Q_RAW(1,2)), % total prec less than total runoff.
        disp(['<< Total available precip (',num2str(Hin.P_RAW(end,2)), ...
            'inches) less than total runoff (',num2str(Hin.Q_RAW(end,2)),'inches) - SKIPPING >>']);
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end

    % NOT DONE YET
    %% check accumulated precip is always larger than accumulated flow...
    %disp('    Checking precip/flow volume for temporal consistency...');
    %vol_check = check_QP_vol(Hin);
    
    % run model...
    [err] = do_run(Hin); % run model.
    if err,
        disp('<< ERROR ENCOUNTERED RUNNING MODEL - SKIPPING >>');
        [Hin] = write_out(Hin,'null'); % write null results to output file.
        continue; % go on to next data file.
    end

    disp(['DONE WITH RUN ',num2str(run),' (APPROXIMATELY ',num2str(100*run_count/n_run,3),'% DONE)']);
end

%--------------------------------------------------------------------------
% DONE.
disp('==============================================================');
disp('DONE WITH BATCH RUN.');

run_time = toc; % stop clock.
disp(['RUN COMPLETED AT: ',datestr(now)]);
disp(['TOTAL RUNTIME: ',sec2hhmmss(run_time)]);

diary off; % stop log file.

return;


%==========================================================================
% AUXILARY FUNCTIONS

%--------------------------------------------------------------------------
% GET WATERSHED CHARACTERISTICS FOR A PARTICULAR STAID
function [AREA,LENGTH,SLOPE,DEVEL] = lookup_ws_char(ws_char,staid)
% ws_char = [STAIDS,AREAS,LENGTHS,SLOPES,DEVELS]
index = (ws_char(:,1)==staid);
if sum(index) == 0, % staid not in list.
    AREA   = [];
    LENGTH = [];
    SLOPE  = [];
    DEVEL  = [];
else
    AREA   = ws_char(index,2);
    LENGTH = ws_char(index,3);
    SLOPE  = ws_char(index,4);
    DEVEL  = ws_char(index,5);
end
return;

%--------------------------------------------------------------------------
% GET C0 FOR A PARTICULAR STAID
function c0 = lookup_c0(staid_c0,staid)
% staid_c0 = [STAIDS,c0(1),c0(2),...]
index = (staid_c0(:,1)==staid);
if sum(index) == 0, % staid not in list.
    c0 = [];
else
    c0 = staid_c0(index,2:end);
end
return;

%--------------------------------------------------------------------------
% READ DATA FILES
function [A,err] = read_data(fname)
err = 0;
try
    % NOTES:
    %   data file can have variable number of header lines which are commented with leading '#'.
    %   1st non-comment line is text headers - read all data as strings to handle uncommented text header line.
    %   files can have multiple columns - dates are 1st col and data are last col, allow up to 9 cols.
    
    fid = fopen(fname);
    data = textscan(fid,'%s %s %s %s %s %s %s %s %s %s','commentStyle','#'); % ignore comment ('#') lines.
    fclose(fid);
    
    % if last col is not empty, there are too many cols in the file...
    if ~strcmp(data{end}{1},''), % last col not empty.
        A = [];
        err = 1;
        return;
    end
    
    % convert cell string dates to char array, ignoring header text...
    data1 = char(data{1}{2:end,:});
    
    % find which column is last non-empty column...
    for col = 2:10,
        if strcmp(data{col}{1},''), % found empty column.
            data2 = data{col - 1}; % last non-empty column.
            break;
        end
    end

    % convert string values to numbers, ignoring header text.
    vals = [];
    for i = 2:length(data2),
        vals = [vals;str2double(data2{i,:})];
    end
    
    % put serial dates and values in array.
    A = [parse_dates(data1),vals]; % put in array.
    
% DO THIS ONLY TO PRECIP LATER
%     % trim repeated values from end of array...
%     for i = size(A,1):-1:2,
%         if A(i,2) ~= A(i-1,2), % 2 different values.
%             A = A(1:i,:); % trim repeated values.
%             break;
%         end
%     end
    
%     % BUG CHECK:
%     assignin('base','data',data);
%     assignin('base','A',A);
%     error('!!')
    
catch
    A = [];
    err = 1;
end
return;

%--------------------------------------------------------------------------
% CONVERT DATA FILE DATE STAMPS TO SERIAL DATES
function [serial_date] = parse_dates(date_str)
% converts a time stamp string into a serial date. 2 possible formats:
%   (1) 'MM/DD/YYYY@HH:MM:SS'  <== 19 chars
%   (2) 'MM/DD/YYYY@HH:MM'     <== 16 chars
mo = str2num(date_str(:,1:2));
dy = str2num(date_str(:,4:5));
yr = str2num(date_str(:,7:10));
hr = str2num(date_str(:,12:13));
mn = str2num(date_str(:,15:16));
if size(date_str,2) == 19, % assume case 1.
    sc = str2num(date_str(:,18:19));
elseif size(date_str,2) == 16, % assume case 2.
    sc = zeros(size(date_str,1),1);
end
serial_date = datenum(yr,mo,dy,hr,mn,sc);
return;

%--------------------------------------------------------------------------
% CONVERT SECONDS TO '### hours ## minutes ## seconds' FORMAT
function hhmmss = sec2hhmmss(sec)

dec_hh = sec/(60*60); % decimal hours.
hh = floor(dec_hh); % hours.
dec_mm = 60*(dec_hh - hh); % decimal minutes remaining.
mm = floor(dec_mm); % minutes.
dec_ss = 60*(dec_mm - mm); % decimal seconds remaining.
ss = floor(dec_ss); % seconds.
hhmmss = [num2str(hh),' hours ',num2str(mm),' minutes ',num2str(ss),' seconds'];
return;



