function [P] = calc_P(Pt,Pd)
% USAGE: [P] = calc_P(Pt,Pd)
%
% DESCRIPTION: This function generates synthetic rainfall time-series.
%
% INPUT:
%   Pt - total rainfall of storm [inches].
%   Pd - total duration of storm [hrs].
%
% OUTPUT:
%   P - rain time-series [inches in 5min increments for duration Pd].
%
% NOTES: The following synthetic precip relation is used:
%   P(t) = (Pt)(t/Pd)^b exp[c-(t/Pd)]
%   where b and c are set parameters. This gives the accumulated rainfall;
%   it is then un-accumulated to get the time-series.
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: [P] = calc_P(Pt,Pd)';

% check arguments...
if nargin ~= 2,
    disp(usage);
    error('Incorrect number of input arguments.');
end

% set parms...
b = 0.7679;
c = 0.1436;

% define synthetic precip function...
accum_p = @(t) Pt.*((t./Pd).^b).*exp(c.*(1-(t./Pd))); % p=[inches], t=[hrs]

% get time vector to use and apply...
t = (0:5:(60*Pd))'; % [mins, in 5min increments]
t = t./60; % put back in [hrs]
accum_P = accum_p(t); % accumulated [inches, 5min increments]

% un-accumulate...
P = unaccum(accum_P); % [inches, 5min increments]

% % for testing...
% plot(t,accum_P); hold on;
% bar(t,P,'r');
% volume = sum(P) % should equal Pt.

return;
