function obj = calc_obj(Hin,c)
% USAGE: obj = calc_obj(Hin,c)
%
% DESCRIPTION: This function calculates flow based on the unit hydrograph method.
%              The results are then compared to the observed, and an
%              objective function is then applied to find a metric to
%              evaluate model performance, which can then be minimized to
%              optimize the model.
%
% INPUT:
%   Hin        - data structure containing run parameters and output.
%   c          - loss function parameters (scalar or vector).
%
% OUTPUT:
%   obj - return value of the objective function gauging the performance
%         of the model. Minimize this to optimize the model.
%
% NOTES: Hin.check_flat checks for flat-line models and returns obj=realmax
% if one is encountered. This way, that c will not be chosen as the
% minimum. If you toggle this, you must redefine the OBJ function to update
% the value. IE:
%       Hin.check_flat = 1; % check for flat-line models.
%       Hin.OBJ = @(c) calc_obj(Hin,c); % update OBJ.
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: obj = calc_obj(Hin,c)';

% check arguments...
if nargin ~= 2,
    disp(usage);
    error('Incorrect number of input arguments.');
end
if ~isstruct(Hin),
    error('Input argument ''Hin'' must be a data structure.');
end
if ~isnumeric(c),
    error('Input argument ''c'' must be numeric.');
end

%--------------------------------------------------------------------------
% RUN MODEL
[Hin,err] = run_model(Hin,c);
if err, % encountered memory problem (shouldn't happen because should be caught during optimization above)
    obj = NaN;
    return; % skip and go on to next run.
end

if Hin.check_flat, % see if got flat-line result...
    if sum(Hin.V_SIM) == 0, % FLAT-LINE RESULT!
        disp('      * Model flat-lined! Retrying...');
        obj = realmax; % assign very large number to obj so this c will not be chosen.
        return;
    end
end

% %***** TEST
% if abs(sum(Hin.RAIN_XS) - Hin.V_OBS(end))/Hin.V_OBS(end) > 1e-2, % no relative volume match.
% %     disp('      * No volume match. Retrying...');
%     obj = realmax; % assign very large number to obj so this c will not be chosen.
%     return;
% end
% %************

%--------------------------------------------------------------------------
% evaluate objective function specified...
obj = Hin.obj_fun(Hin.V_SIM(end),Hin.V_OBS(end), Hin.V_SIM,Hin.V_OBS, Hin.Q_SIM,Hin.Q_OBS);

return;

