function [Vt,Vtot] = calc_vol(Q,area)
% USAGE: [Vt,Vtot] = calc_vol(Q,area)
%
% DESCRIPTION: This function calculates the accumulated volume of water
%   (as a function of time and total) given by a hydrograph time-series.
%   A mid-point intergration method is used.
%
% INPUT:
%   Q - column vector containing hydrograph time-series [CFS in 5min intervals].
%   area - drainage area of watershed [mi^2].
%
% OUTPUT:
%   Vt   - volume time-series [inches].
%   Vtot - (optional) total volume [inches].
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: [Vt,Vtot] = calc_vol(Q,area)';

% check arguments...
if nargin ~= 2,
    disp(usage);
    error('Incorrect number of input arguments.');
end

Vt = NaN(size(Q)); % initialize.
Vtot = 0; % initialize.

% get contribution from left endpoint...
Vtot = Vtot + Q(1)*2.5*60; % [ft^3]
Vt(1) = Vtot;

% get contributions from the middle values...
for i = 2:(length(Q)-1),
    Vtot = Vtot + 5*60*Q(i); % [ft^3]
    Vt(i) = Vtot;
end

% get contribution from right endpoint...
Vtot = Vtot + Q(end)*2.5*60; % [ft^3]
Vt(end) = Vtot;

% use area to convert to inches...
conv = 12/((5280^2)*area); % [ft^3 => in]
Vtot = conv*Vtot; % [in]
Vt   = conv*Vt;   % [in]

return;
