function [err] = do_run(Hin)
% USAGE: [err] = do_run(Hin)
%
% DESCRIPTION: This function runs a single unit hydrograph optimization run.
%
% INPUT:
%   Hin - data structure containing the following variables:
%     use_synthP - indicates whether to use synthetic or raw rainfall data
%          0 = use raw rainfall
%          1 = use synthetic rainfall
%     Lfun - string indicating which loss function to use:
%          'L1': 1-parameter linear,            L(t) = c1
%          'L2': 2-parameter linear,            L(t) = c1*t + c2
%          'L3': 3-parameter quadratic,         L(t) = c1*t^2 + c2*t + c3
%          'E1': 1-parameter exponential decay, L(t) = exp(-c1*t)
%          'E2': 2-parameter exponential decay, L(t) = c1*exp(-c2*t)
%          'E3': 3-parameter exponential decay, L(t) = c1*exp( -(c2*t + c3))
%          'E4': 4-parameter exponential decay, L(t) = c1*exp( -(c2*t + c3)) + c4
%          'P1': 1-parameter proportional loss, P(t) = P(t) - c1*P(t)
%          'A1': 1-parameter init. abstraction, P(t) = P(t) - init.abs(c1*Ptot)
%    c0 - scalar or vector specifying the initial loss function parameters, as defined above.
%    obj - integer indicating objective function to use:
%          1 = minimize total volume residual.
%          2 = minimize continuous flow residuals.
%          3 = minimize continuous volume residuals.
%          4 = continuous flow least-squares.
%          5 = continuous volume least-squares.
%    SLOPE  - channel slope                        [dimensionless]
%    LENGTH - channel length                       [mi]
%    DEVEL  - development status                   [0=undeveloped, 1=developed]
%    AREA   - drainage area                        [mi^2]
%    Q_RAW  - raw rain read from input file        [inches]
%    P_RAW  - raw accumulated flow from input file [inches]
%    outfile - string name of the output to append model results to.
%    hydro_file - string name of hydrograph file - used to get staid and dates.
%    save_pdf - indicates whether to create and save PDF of run:
%               0 = do not create and save PDF.
%               1 = create and save PDF.
%
% OUTPUT:
%   err - error flag: 0=good run, 1=encountered an error.
%
% Written by Joe Vrabel, USGS May2006

warning('off', 'stats:regress:RankDefDesignMat'); % sometimes given by 'regress' for fit stats.

err = 0;
usage = 'USAGE: [err] = do_run(Hin)';

%--------------------------------------------------------------------------
% PROCESS RAW P AND Q DATA...

%---------------
% PRECIP

% linear interpolate raw accum rain into 5min increments...
Train = [Hin.P_RAW(1,1):1/288:Hin.P_RAW(end,1)]'; % time vector of raw rain in 5min intervals.
Hin.ACCUM_RAIN = interp1(Hin.P_RAW(:,1),Hin.P_RAW(:,2),Train, 'linear');

% trim leading and trailing tails of interpolated precip...
trimmed_rain = trim_precip([Train,Hin.ACCUM_RAIN],Hin.P_trim1,Hin.P_trim2);
Hin.ACCUM_RAIN = trimmed_rain(:,2); % extract values only.
Hin.event_start = trimmed_rain(1,1); % start of event after trimming.

% un-accumulate rain [inches].
Hin.RAIN = unaccum(Hin.ACCUM_RAIN);

% get actual rain to use in model (ie: synth or actual)...
Hin.RAIN_VOL = sum(Hin.RAIN); % total rainfall [inches].
Hin.RAIN_DUR = (5/60)*(length(Hin.RAIN)-1); % rainfall duration, assumes given in 5min increments [hrs].
if Hin.use_synthP,
    [Hin.RAIN_USE] = calc_P(Hin.RAIN_VOL,Hin.RAIN_DUR); % generate synthetic rain with same volume and duration [inches in 5min increments].
else
    Hin.RAIN_USE = Hin.RAIN; % use actual rain.
end

%---------------
% FLOW
% linear interpolate raw accum Q into 5min increments...
Hin.V_OBS = interp1(Hin.Q_RAW(:,1),Hin.Q_RAW(:,2),Hin.Q_RAW(1,1):1/288:Hin.Q_RAW(end,1), 'linear')';

% subtract offset from volume so it starts at 0...
Hin.V_OBS = Hin.V_OBS - Hin.V_OBS(1);

% calculate actual discharge (non-uniform spacing) using "bRcT" GUGAS method [CFS]...
Hin.Q_OBS  = (1/24)*inhr2cfs(bRcT_diff(Hin.Q_RAW(:,1),Hin.Q_RAW(:,2)),Hin.AREA);

% interpolate actual discharge to uniform 5min increments...
Hin.Q_OBS = interp1(Hin.Q_RAW(:,1),Hin.Q_OBS, Hin.Q_RAW(1,1):1/288:Hin.Q_RAW(end,1), 'linear')';

% find pad time in 5min increments so sim and obs vectors line up in time after applying model ...
Hin.pad = round((24*12)*(trimmed_rain(1,1) - Hin.Q_RAW(1,1))); % need pad using trimmed rain times.
% Hin.pad = round((24*12)*(Hin.P_RAW(1,1) - Hin.Q_RAW(1,1))); % old wrong way.

%--------------------------------------------------------------------------
% OPTIMIZATION APPLYING LOSS MODEL

% create function handle to the objective function...
Hin.check_flat = 1; % check for flat-line models.
Hin.OBJ = @(c) calc_obj(Hin,c);

% check to see if memory is exceeded...
% (OBJ returns NaN if the convolution matrix requires more than max_array_size_MB)
if isnan(Hin.OBJ(Hin.c0)), % max allowable array size exceeded.
    err = 1;
    return;
end

% use the specified optimization method...
if strcmpi(Hin.opt_meth,'none'), % no optimization - set Copt to c0...
    disp('    No optimization performed (c0 used in 1 model run).');
    Hin.c_opt = Hin.c0;
    %!!!!!
    Hin.c_opt(1) = Hin.c_opt(1)/sum(Hin.RAIN_USE); % convert input init.abstract from inches ==> fraction of total rain.
    %!!!!!
    Hin.obj_opt = NaN; % need dummy objective function value.
else
    disp('    Performing optimization...');
    if strcmpi(Hin.opt_meth,'grid_search'), % 'adaptive' grid search method.

        n_pass = length(Hin.grid_tols); % number of 'zoom' passes.

        % initialize debug figure if in DB mode...
        if Hin.db_obj,
            scr = get(0,'ScreenSize'); % [left bottom width height]
            figure('Position',[scr(1),scr(2)+0.01,scr(3),0.94*scr(4)], ...
                   'NumberTitle','off','Name','DEBUG: grid search pass results');
            curr_subplot = 0;
        end


        % initialize parm space search bounds for 1st pass...
        min_x = Hin.min_c(1); max_x = Hin.max_c(1);
        min_y = Hin.min_c(2); max_y = Hin.max_c(2);

        % perform passes...
        for pass = 1:n_pass,

            % display pass info...
            mag = ((Hin.max_c(1)-Hin.min_c(1))*(Hin.max_c(2)-Hin.min_c(2))) / ...
                ((max_x-min_x)*(max_y-min_y)); % zoom (magnification) factor relative to original domain.
            disp(['      * Pass #',num2str(pass),' of ',num2str(n_pass),', ', ...
                'p-space: [',num2str(min_x),',',num2str(max_x),']x[',num2str(min_y),',',num2str(max_y),'], ', ...
                'zoom factor: ',num2str(mag)]);

            % grid the parameter space with the current search bounds...
            x = linspace(min_x,max_x,Hin.n_c1(pass));
            y = linspace(min_y,max_y,Hin.n_c2(pass));

            % initialize grid array if debugging...
            if Hin.db_obj,
                Psp = NaN(length(y),length(x)); % stores whether [c1,c2] meets tol.
            end

            % initialize array storing valid c values for each pass...
            valid = []; % valid parm pairs match rel. sim/obs volume within tol.

            % do 1 pass...
            for i = 1:length(y), % going down.
                for j = 1:length(x), % going across.

                    % assign loss function parameter pair...
                    c = [x(j),y(i)];

                    % get the excess rain for the current parameter pair...
                    [RAIN_XS,loss] = precip_loss(Hin.RAIN_USE,Hin.Lfun,c,Hin.no_neg_P,Hin.no_neg_loss);

                    % calculate the relative sim/obs volume error...
                    relerr = abs(sum(RAIN_XS) - Hin.V_OBS(end))/Hin.V_OBS(end);

                    % see if rel.err. meets current tol...
                    if relerr <= Hin.grid_tols(pass), % get vol match within tol with this [c1,c2].
                        valid = [valid;[c,NaN]]; % append to valid list - last column for obj function value found later.
                        % if debug run, fill grid array with result...
                        if Hin.db_obj,
                            Psp(i,j) = relerr;
                        end
                    else % no vo match - [c1,c2] no good, don't add to valid list.
                        % if debug run, fill grid array with 0...
                        if Hin.db_obj,
                            Psp(i,j) = 0;
                        end
                    end
                end
            end

            % produce plot if debugging...
            if Hin.db_obj,
                % error check...
                if sum(isnan(Psp)) ~= 0, % at least one element of Psp not assigned.
                    error('At least one element of Psp not assigned.');
                end

                % plot parm space...
                curr_subplot = curr_subplot + 1;
                subplot(ceil(sqrt(n_pass)),ceil(sqrt(n_pass)),curr_subplot);
                map = colormap('gray'); map = flipud(map); % use gray scale, but with 0=>1 = white=>black.
                map(1,:) = [0.2,0.2,0.5]; % set 0 (non-valid parms) to green.
                imagesc(x,y,Psp); colormap(map); colorbar;
                xlabel('C_1 (UNADJUSTED)');
                ylabel('C_2 (UNADJUSTED)');
                title(['\bf','PASS#',num2str(pass), ...
                          '  n\_c1=',num2str(Hin.n_c1(pass)), ...
                          '  n\_c2=',num2str(Hin.n_c2(pass)), ...
                            '  tol=',num2str(Hin.grid_tols(pass)), ...
                            '  mag=',num2str(round(mag))]);
                drawnow;
            end

            % if no valid parms found in current parm space, method failed.
            if isempty(valid),
                disp('    Method failed - no valid parameters found in current grid pass.');
                err = 1;
                return; % go back and skip this run.
            end

            % reset parm space search bounds to 'zoom in' on valid region...
            curr_x_range = max_x - min_x; % x-range of current parameter space.
            min_x = min(valid(:,1));
            max_x = max(valid(:,1));
            % handle if min = max...
            if min_x == max_x, % set x's to +-10% of current x-range.
                min_x = min_x - 0.1*curr_x_range;
                max_x = max_x + 0.1*curr_x_range;
                if min_x < Hin.min_c(1), % wandered out of allowed region.
                    min_x = Hin.min_c(1);
                end
                if max_x > Hin.max_c(1), % wandered out of allowed region.
                    max_x = Hin.max_c(1);
                end
            end

            curr_y_range = max_y - min_y; % y-range of current parameter space.
            min_y = min(valid(:,2));
            max_y = max(valid(:,2));
            % handle if min = max...
            if min_y == max_y, % set y's to +-10% of current y-range.
                min_y = min_y - 0.1*curr_y_range;
                max_y = max_y + 0.1*curr_y_range;
                if min_y < Hin.min_c(2), % wandered out of allowed region.
                    min_y = Hin.min_c(2);
                end
                if max_y > Hin.max_c(2), % wandered out of allowed region.
                    max_y = Hin.max_c(2);
                end
            end

        end

        disp(['    There are ',num2str(size(valid,1)),' final parameter pair canidates.']);

        % calc obj function for all final parm canidates...
        disp( '    Evaluating objective function for final canidates...');
        for i = 1:size(valid,1),
            valid(i,3) = calc_obj(Hin,[valid(i,1),valid(i,2)]);
        end

        valid(valid(:,3)==realmax,:) = []; % get rid of any realmax's from flat-lining.
        if isempty(valid), % all final canidates found were realmax's!
            disp('    Method failed - all final valid parameters found are realmax.');
            err = 1;
            return; % go back and skip this run.
        end

        Hin.c_opt   = valid(valid(:,3)==min(valid(:,3)),[1 2]); % set parms for smallest obj func value.
        Hin.obj_opt = valid(valid(:,3)==min(valid(:,3)),3);     % set smallest obj func value.

        if (size(Hin.c_opt,1) ~= 1) || (size(Hin.obj_opt,1) ~= 1), % did not find a unique final c_opt or obj_opt value.
            disp(['    A unique parameter pair was not found - mean of ',num2str(size(Hin.c_opt,1)),' values used.']);
            Hin.c_opt   = mean(Hin.c_opt);   % take mean of remaining parms.
            Hin.obj_opt = mean(Hin.obj_opt); % take mean of remaining values.
        end
        disp(['    The following parameter pair meets the final relative volume match error tolerance of ', ...
            num2str(Hin.grid_tols(end)),' and has the smallest objective function value:']);
        disp(['      c_opt found (unadjusted): ',num2str(Hin.c_opt)]);

    elseif strcmpi(Hin.opt_meth,'fmin') || strcmpi(Hin.opt_meth,'fbnd'), % fminsearch bounded/unbounded method.

        % optimize parameter by minimizing the objective function in parameter space...
        % set optimization options:
        options = optimset('fminsearch');
        options.Display = 'notify'; % 'off' displays no output.
                                    % 'iter' displays output at each iteration.
                                    % 'final' displays just the final output.
                                    % 'notify' (default) displays output only if the function does not converge.
        options.FunValCheck = 'on'; % checks whether objective function values are valid.
                                    % 'on' displays a warning when the objective function returns a value that is complex, Inf or NaN.
                                    % 'off' (the default) displays no warning.
        options.MaxIter = Hin.fmin_max_its; % maximum number of iterations allowed.
        options.MaxFunEvals = 2*Hin.fmin_max_its; % maximum number of function evaluations allowed - usually twice number of iterations.
        % options.OutputFcn = ; % specify a user-defined function that the optimization function calls at each iteration.
        options.TolFun = Hin.fmin_obj_tol; % termination tolerance on the objective function.
        % options.TolX = ; % termination tolerance on the c's (MatLab default = 1e-4).

        % optimize...
        disp(     '    Performing optimization...');
        n_monte = 0; % monte carlo iteration count.
        for i = 1:Hin.max_monte + 1,
            n_monte = n_monte + 1; % increment count.
            % see if max_monte exceeded...
            if n_monte > Hin.max_monte, % monte carlo operation did not break stationary point in max_monte iterations.
                disp('    Monte Carlo iterations did not break stationary point in maximum iterations allowed.');
                err = 1;
                return;
            end

            if strcmpi(Hin.opt_meth,'fbnd'), % perform bounded fminsearch.
                [Hin.c_opt,Hin.obj_opt,exitflag,opt_info] = fminsearchbnd(Hin.OBJ, Hin.c0, Hin.min_c, Hin.max_c, options); % optimize.
            else % perform unrestricted search.
                [Hin.c_opt,Hin.obj_opt,exitflag,opt_info] = fminsearch(Hin.OBJ, Hin.c0, options); % optimize.
            end

            % check for stationary point...
            if Hin.c_opt ~= Hin.c0, % not a stationary point - done.
                break;
            else % randomize a new c0 within [c_min,c_max].
                Hin.c0 = (Hin.max_c - Hin.min_c).*rand(size(Hin.c0)) + Hin.min_c;
                disp('      * Stationary point encountered');
                disp(['         Resetting random c0 = ',num2str(Hin.c0),' (attempt ',num2str(n_monte),' out of ',num2str(Hin.max_monte),')']);        
            end
        end

        if exitflag == 0,
            disp('    ... maximum number of function evaluations or iterations reached without meeting convergence criteria.');
            err = 1;
            return;
        elseif exitflag == -1,
            disp('    ... algorithm was terminated by the output function without solution convergence.');
            err = 1;
            return;
        end
        disp(opt_info.message);
        disp( '    Optimization details:');
        disp(['      * Algorithm used: ',opt_info.algorithm]);
        disp(['      * Number of function evaluations: ',num2str(opt_info.funcCount)]);
        disp(['      * Number of iterations: ',num2str(opt_info.iterations)]);
        disp(['      * Final value of objective function: ',num2str(Hin.obj_opt)]);
        disp( '      * Optimal parameters found (unadjusted):');
        for i = 1:length(Hin.c_opt),
            disp(['         c_opt(',num2str(i),') = ',num2str(Hin.c_opt(i))]);
        end
    end

end

%--------------------------------------------------------------------------
% REDO ALL CALULATIONS WITH OPTIMIZED RESULTS
% generate optimized rain series found...
disp('    Running model with optimized results...');
[Hin,err] = run_model(Hin,Hin.c_opt);
if err, % encountered memory problem (shouldn't happen because should be caught during optimization above)
    err = 1;
    return; % skip and go on to next run.
end

%--------------------------------------------------------------------------
% CONVERT SOME C_OPT's FOR DESIRED OUTPUT
switch upper(Hin.Lfun),
    case {'A1','AC2','AP2'} % involve init.abs.
        Hin.c_opt_out = Hin.c_opt;
        Hin.c_opt_out(1) = Hin.c_opt(1)*sum(Hin.RAIN_USE); % convert percent init.abs. to actual inches abstracted.
    otherwise
        Hin.c_opt_out = Hin.c_opt; % no changes.
end

%--------------------------------------------------------------------------
% APPEND OUTPUT TO OUTFILE
disp('    Writing run results to output...');
[Hin] = write_out(Hin);

%--------------------------------------------------------------------------
% PLOT OPTIMIZED RESULTS (OPTIONAL)
if Hin.save_pdf,
    disp('    Plotting model results...');
    plot_out(Hin);
end

%--------------------------------------------------------------------------
% DEBUG OBJ PLOT (OPTIONAL)
if Hin.db_obj,
    disp('    DEBUG: Calculating and plotting objective function space and minimum found...');
    plot_obj(Hin);
end

return;


