function [n_peaks] = find_n_peaks(v,pos_neg,rel_tol)
% USAGE: [n_peaks] = find_n_peaks(v,'pos_neg',[rel_tol])
%
% DESCRIPTION: This function finds the number of positive or negative peaks
%   in a given input vector.
%
% INPUT:
%   v - (numeric vector) input data.
%
% OUTPUT:
%   n_peaks   - (numeric scalar) number of peaks in the input vector.
%   'pos_neg' - (string) can be 1 of 3 values:
%                * 'pos' finds the number of positive peaks
%                * 'neg' finds the number of negative peaks
%                * 'all' finds both the negative and positive peaks
%   rel_tol   - (optional, numeric scalar) tolerance criteria for a 'peak';
%               ie: |changes| < tol*(max(v)-min(v)) will be ignored. This
%               is used to filter out small peaks relative to the total
%               range of the data.
%               Default: eps (machine epsilon; no effective filtering).
%
% Written by: Joe Vrabel, USGS June2006

usage = 'USAGE: [n_peaks] = find_n_peaks(v,''pos_neg'',[rel_tol])';

% check input args...
if (nargin < 2) || (nargin > 3),
    disp(usage);
    error('Incorrect number of input arguments.');
end
if nargin == 2, % default rel_tol.
    rel_tol = eps;
end
if ~isnumeric(v),
    disp(usage);
    error('Input argument ''v'' must be a numeric vector.');
elseif ~isvector(v) || length(v) < 2,
    disp(usage);
    error('Input argument ''v'' must be a numeric vector with at least 2 elements.');
end
if ~isnumeric(rel_tol),
    disp(usage);
    error('Input argument ''rel_tol'' must be a numeric scalar.');
elseif numel(rel_tol) ~= 1,
    disp(usage);
    error('Input argument ''rel_tol'' must be a numeric scalar.');
end
if ~ischar(pos_neg),
    disp(usage);
    error('Input argument ''pos_neg'' must be a string set to ''pos'', ''neg'' of ''all''.');
end

% get backwards differences of data...
d = diff(v);

% get rid of no change (flat) regions - need to handle round off error.
% changes of 0.0001% of range of data is ignored.
d(abs(d) < rel_tol*(max(v)-min(v))) = [];

if isempty(d), % no peaks - done.
    n_peaks = 0;
    return;
end

% fill vector of sign changes in differences...
sign_change = sign(d(1));
for i = 2:length(d),
    if sign(d(i)) ~= sign(d(i-1)), % sign flip.
        sign_change = [sign_change,sign(d(i))];
    end
end

% find number of peaks...
sign_change = num2str(sign_change);
sign_change(sign_change==' ') = []; % get rid of spaces.
switch lower(pos_neg),
    case 'pos' % find number of occurances of the '1-1' pattern, indicating a positive peak.
        n_peaks = length(strfind(sign_change,'1-1'));
    case 'neg' % find number of occurances of the '-11' pattern, indicating a positive peak.
        n_peaks = length(strfind(sign_change,'-11'));
    case 'all' % find number of occurances of both '-11' and '1-1' patterns.
        n_peaks = length(strfind(sign_change,'1-1')) + length(strfind(sign_change,'-11'));
    otherwise
        disp(usage);
        error('Input argument ''pos_neg'' must be a string set to ''pos'',''neg'', or ''all''.');
end

% % bug check:
% plot(v),title([sign_change,': ',num2str(n_peaks)]);
% assignin('base','v',v);
% error('asd')

return;

