function [x,fval,exitflag,output]=fminsearchbnd(fun,x0,LB,UB,options,varargin)
% USAGE: [x,fval,exitflag,output] = fminsearchbnd(fun,x0,LB,UB,options,varargin)
%
% DESCRIPTION: This function finds the minimum of a multivariable function with bounded constraints.
%   It uses FMINSEARCH, but with bound constraints by transformation.
%
% INPUT:
%   fun      - function handle to the function to minimize.
%   x0       - vector of the starting guess.
%   LB       - lower bound vector or array, must be the same size as x0.
%              If no lower bounds exist for one of the variables, then supply -inf for that variable.
%              If no lower bounds at all, then LB may be left empty.
%              Variables may be fixed in value by setting the corresponding lower and upper bounds to exactly the same value.
%   UB       - upper bound vector or array, must be the same size as x0.
%              If no upper bounds exist for one of the variables, then supply +inf for that variable.
%              If no upper bounds at all, then UB may be left empty.
%              Variables may be fixed in value by setting the corresponding lower and upper bounds to exactly the same value.
%   options  - structure of options to use (see OPTIMSET).
%
% OUTPUT:
%   x        - the minimum value found.
%   fval     - the function value at x.
%   exitflag - flag that describes the exit condition (see FMINSEARCH).
%   output   - structure containing algorithm details (see FMINSEARCH).
%
% NOTES:
%  If options is supplied, then TolX will apply to the transformed
%  variables. All other FMINSEARCH parameters should be unaffected.
%
%  Variables which are constrained by both a lower and an upper
%  bound will use a sin transformation. Those constrained by
%  only a lower or an upper bound will use a quadratic
%  transformation, and unconstrained variables will be left alone.
%
%  Variables may be fixed by setting their respective bounds equal.
%  In this case, the problem will be reduced in size for FMINSEARCH.
%
%  The bounds are inclusive inequalities, which admit the
%  boundary values themselves, but will not permit ANY function
%  evaluations outside the bounds. These constraints are strictly
%  followed.
%
%  If your problem has an EXCLUSIVE (strict) constraint which will
%  not admit evaluation at the bound itself, then you must provide
%  a slightly offset bound. An example of this is a function which
%  contains the log of one of its parameters. If you constrain the
%  variable to have a lower bound of zero, then FMINSEARCHBND may
%  try to evaluate the function exactly at zero.
%
% Written by: John D'Errico (woodchips@rochester.rr.com) July2006.
% Modified by: Joe Vrabel Aug2006


% check input...
if nargin < 2,
    error('Not enough input arguments.');
end

%--------------------------------------------------------------------------
% PREP INPUT
xsize = size(x0);   % original size of x0.
x0    = x0(:);      % put x0 in a column vector.
n     = length(x0); % number of variabls.

% set LB / UB if not specified...
if (nargin < 3) || isempty(LB) % LB not specified.
    LB = repmat(-inf,n,1); % set to [-inf;-inf;...]
else
    LB = LB(:); % put in a column vector.
end
if (nargin < 4) || isempty(UB) % UB not specified.
    UB = repmat(inf,n,1); % set to [-inf;-inf;...]
else
    UB = UB(:); % put in a column vector.
end

% check size consistency...
if (n ~= length(LB)) || (n ~= length(UB))
    error 'x0 is incompatible in size with either LB or UB.'
end

% set default options if necessary...
if (nargin < 5) || isempty(options), % options not specified.
    options = optimset('fminsearch');
end

% put parameters into a structure to pass around...
params.args = varargin;
params.LB   = LB;
params.UB   = UB;
params.fun  = fun;
params.n    = n;

%--------------------------------------------------------------------------
% TRANSFORM VARIABLES

% set boundary class codes for each x0 element:
% 0 --> unconstrained variable
% 1 --> lower bound only
% 2 --> upper bound only
% 3 --> dual finite bounds
% 4 --> fixed variable
params.BoundClass = zeros(n,1);
for i = 1:n,
    k = isfinite(LB(i)) + 2*isfinite(UB(i));
        params.BoundClass(i) = k;
    if (k == 3) && (LB(i) == UB(i))
        params.BoundClass(i) = 4;
    end
end

% transform starting values into their unconstrained surrogates.
% also check for infeasible starting guesses (ie: starting guesses
% outside specified bounds)...
x0u = x0; % x0u = transformed variable to minimize.
k = 1;
for i = 1:n
    switch params.BoundClass(i)
        case 1 % lower bound only.
            if x0(i) <= LB(i) % infeasible starting value. Use bound.
                x0u(k) = 0;
            else % transform.
                x0u(k) = sqrt(x0(i) - LB(i));
            end
            k = k + 1; % increment k.
      
        case 2 % upper bound only.
            if x0(i) >= UB(i) % infeasible starting value. use bound.
                x0u(k) = 0;
            else % transform.
                x0u(k) = sqrt(UB(i) - x0(i));
            end
            k = k + 1; % increment k.

        case 3 % lower and upper bounds.
            if x0(i) <= LB(i) % infeasible starting value.
                x0u(k) = -pi/2;
            elseif x0(i) >= UB(i) % infeasible starting value.
                x0u(k) = pi/2;
            else % transform.
                x0u(k) = 2*(x0(i) - LB(i))/(UB(i)-LB(i)) - 1;
                % shift by 2*pi to avoid problems at zero in fminsearch.
                % otherwise, the initial simplex is vanishingly small.
                x0u(k) = 2*pi + asin(max(-1,min(1,x0u(k))));
            end
            k = k + 1; % increment k.
      
        case 0 % unconstrained variable. x0u(i) is set.
            x0u(k) = x0(i); % do not transform.
            k = k + 1; % increment k.
      
        case 4 % fixed variable.
            % drop this variable before fminsearch sees it.
            % k is not incremented for this variable.
            % (ie: not incrementing skips this variable.)
    end 
end

% if any of the unknowns were fixed, then we need to shorten x0u now...
if k <= n
    x0u(k:n) = [];
end

%--------------------------------------------------------------------------
% SPECIAL CASE: all variables fixed - no optimization needed.
if isempty(x0u) % All variables were fixed.
    % quit immediately: set the appropriate parameters and return.

    % undo the variable transformations into the original space...
    x = xtransform(x0u,params);

    % reshape x to original dimensions...
    x = reshape(x,xsize);

    % set fval with the final value...
    fval = feval(params.fun,x,params.args{:});

    % a solution was found...
    exitflag = 1;

    output.iterations = 0; % no fminsearch iterations.
    output.funcount   = 1; % just the 1 function evaluation.
    output.algorithm  = 'fminsearchbnd';
    output.message    = 'All variables were held fixed by the applied bounds';

    return % return with no call at all to fminsearch.
end

%--------------------------------------------------------------------------
% NORMAL OPTIMIZATION

% now we can call fminsearch, but with our own intra-objective function...
[xu,fval,exitflag,output] = fminsearch(@intrafun,x0u,options,params);

% undo the variable transformations into the original space...
x = xtransform(xu,params);

% final reshape to original dimensions...
x = reshape(x,xsize);

return;



%==========================================================================
% AUXILARY FUNCTIONS

function fval = intrafun(x,params)
% transforms variables, then calls original function.

% transform....
xtrans = xtransform(x,params);

% and call fun...
fval = feval(params.fun,xtrans,params.args{:});

return;

%==========================================================================
function xtrans = xtransform(x,params)
% converts unconstrained variables into their original domains.

xtrans = zeros(1,params.n);
% k allows some variables to be fixed, thus dropped from the optimization.
k = 1;
for i = 1:params.n
    switch params.BoundClass(i)
        case 1 % lower bound only.
            xtrans(i) = params.LB(i) + x(k).^2;
            k = k + 1;
            
        case 2 % upper bound only.
            xtrans(i) = params.UB(i) - x(k).^2;
            k = k + 1;
            
        case 3 % lower and upper bounds.
            xtrans(i) = (sin(x(k))+1)/2;
            xtrans(i) = xtrans(i)*(params.UB(i) - params.LB(i)) + params.LB(i);
            % just in case of any floating point problems
            xtrans(i) = max(params.LB(i),min(params.UB(i),xtrans(i)));
            k = k + 1;
            
        case 4 % fixed variable, bounds are equal, set it at either bound.
            xtrans(i) = params.LB(i);
            
        case 0 % unconstrained variable.
            xtrans(i) = x(k);
            k = k + 1;
            
    end
end

return;


