function staid_c0 = get_c0(c0file)
% USAGE: c0 = get_c0('c0file')
%
% DESCRIPTION:
%   This function reads an input csv file and returns an array of station ID's and loss parameters.
%
% INPUT:
%   'c0file' - string containing the name of the input csv file.
%              It must be formated as:
%        STAID,   c0(1),            c0(2), ...
%        8042650, 1.04850956631736, 0.785116527668254
%        8042700, 1.13805598997347, 0.638304990808263
%        8048520, 0.64716217106834, 0.38662800689957
%        ... more lines as necessary ...
%
% OUTPUT:
%   staid_c0 - numerical array with the following column vectors:
%     (col1) STAIDS  - USGS site identifier as an integer.
%     (col2) c0(1)   - 1st loss function parameter
%     (col3) c0(2)   - 2nd loss function parameter
%     ... additional columns for all c0's specified ...
%
% Written by Joe Vrabel, USGS Feb2007.

usage = 'USAGE: c0 = get_c0(''c0file''))';

% check input args...
if nargin ~= 1,
    disp(usage);
    error('Incorrect number of input arguments.');
end
if ~ischar(c0file),
    disp(usage);
    error('Input argument ''c0file'' must be a string.');
end
if exist(c0file,'file') ~= 2,
    disp(usage);
    error(['Input file ''',c0file,''' does not exist.']);
end

% read data file...
try
    staid_c0 = csvread(c0file, 1, 0); % skip 1 header line.
catch
    disp(usage);
    error(['Cannot read input file: ''',c0file,'''.']);
end

sortrows(staid_c0,1); % sortby STAID.
if ~isequal(unique(staid_c0(:,1)),staid_c0(:,1)), % duplicate staid in list.
    error('Input data has multiple c0 definitions for one or more stations.');
end

return;

