function [hydro_files,hyeto_files] = get_files(datadir,ws_char)
% USAGE: [hydro_files,hyeto_files] = get_files('datadir',ws_char)
%
% DESCRIPTION:
%   This function finds all hydrograph and hyetograph data files in
%   the input directory and all its subdirectories. Only those that have
%   watershed characteristics defined are output.
%
%   The file names (with out paths) of the hydro_files is sorted to produce
%   a sort key. Since the hydro_files are named as (for example):
%                    "unit_sta08155200_1978_0606.dat"
%   the resulting key is in the sorted by downstream order (station ID)
%   then event date. Then both the full hydro_ and hyeto_files arrays are
%   sorted by this key prior to output.
%
% INPUT:
%   datadir - string denoting the directory to search for hydrograph and
%             hyetograph data files. If omitted, the current working
%             directory is used.
%   ws_char - array containing the watershed characteristics given by:
%             [STAIDS,AREAS,LENGTHS,SLOPES,DEVELS]
%
% OUTPUT:
%   hydro_files - cell string containing the complete path and names of the
%                 hydrograph data files found.
%
%   hyeto_files - cell string containing the complete path and names of the
%                 hyetograph data files found.
%
% NOTES:
%   Hydrograph data files are identified with the naming convention:
%     'unit_sta########_YYYY_MMDD.dat'
%   where ######## is the 8-digit USGS station ID and YYYY, DD, MM are the
%   year, month, and day of the data, respectively.
%   Only those files whose full path contain '_D' and 'hydros' are output.
%
%   Hyetograph data files are identified with the naming convention:
%     'rain_sta########_YYYY_MMDD.dat'
%   where ######## is the 8-digit USGS station ID and YYYY, DD, MM are the
%   year, month, and day of the data, respectively.
%   Only those files whose full path contain '_D' and 'hyetos' are output.
%   
%   Before output, the 2 file lists are sorted and compared; each individual
%   hydro file must be matched with a hyeto file and vice-versa - those
%   with out a partner are not output.
%
% Written by Joe Vrabel, USGS May2006.

usage = 'USAGE: [hydro_files,hyeto_files] = get_files(''datadir'',ws_char)';

% check input arg...
if nargin ~= 2,
    disp(usage);
    error('Incorrect hnumber of input arguments.');
end
if ~ischar(datadir),
    disp(usage);
    error('Input argument ''datadir'' must be a string.');
end
if ~isnumeric(ws_char),
    disp(usage);
    error('Input argument ''ws_char'' must be a numeric array.');
elseif size(ws_char,2) ~= 5,
    disp(usage);
    error('Input argument ''ws_char'' must have 5 columns.');
end
if datadir(end) ~= '\',
    datadir = [datadir,'\']; % add trailing \.
end
if exist(datadir,'dir') ~= 7, % not a directory.
    error(['Data directory specified: ''',datadir,''' does not exist - STOPPING.']);
end

%--------------------------------------------------------------------------
% HYDRO FILES
disp(['* Finding all hydrograph data files in: ',datadir,' ...']);

% create batch file to get and write list of all hydro files...
fid = fopen('001temp.bat','w');
fprintf(fid,'%s\n','@echo off');
fprintf(fid,'%s\n',['dir /s /b ',datadir,'\unit_sta*.dat > 001temp.txt']);
fclose(fid);

% execute batch file...
! 001temp.bat

% read hydro files into cell array...
fid = fopen('001temp.txt','r');
temp = textscan(fid,'%s');
fclose(fid);

% remove files that don't have '_d' in path...
temp = temp{1};
hydro_files = {};
for i = 1:length(temp),
    if ~isempty(findstr(lower(temp{i}),'_d\'));
        hydro_files = [hydro_files;temp{i}];
    end
end

% remove files that don't have 'hydros' in path...
temp = hydro_files;
hydro_files = {};
for i = 1:length(temp),
    if ~isempty(findstr(lower(temp{i}),'\hydros\unit_sta'));
        hydro_files = [hydro_files;temp{i}];
    end
end

hydro_files = sort(hydro_files);

%--------------------------------------------------------------------------
% HYETO FILES
disp(['* Finding all hyetograph data files in: ',datadir,' ...']);

% create batch file to get and write list of all hyeto files...
fid = fopen('001temp.bat','w');
fprintf(fid,'%s\n','@echo off');
fprintf(fid,'%s\n',['dir /s /b ',datadir,'\rain_sta*.dat > 001temp.txt']);
fclose(fid);

% execute batch file...
! 001temp.bat

% read hydro files into cell array...
fid = fopen('001temp.txt','r');
temp = textscan(fid,'%s');
fclose(fid);

% remove files that don't have '_d' in path...
temp = temp{1};
hyeto_files = {};
for i = 1:length(temp),
    if ~isempty(findstr(lower(temp{i}),'_d\'));
        hyeto_files = [hyeto_files;temp{i}];
    end
end

% remove files that don't have 'hyetos' in path...
temp = hyeto_files;
hyeto_files = {};
for i = 1:length(temp),
    if ~isempty(findstr(lower(temp{i}),'\hyetos\rain_sta'));
        hyeto_files = [hyeto_files;temp{i}];
    end
end

hyeto_files = sort(hyeto_files);

%--------------------------------------------------------------------------
% clean up temp files...
if exist('001temp.bat','file') == 2,
    delete('001temp.bat');
end
if exist('001temp.txt','file') == 2,
    delete('001temp.txt');
end

disp(['* ',num2str(length(hydro_files)),' data file pairs found in directory: ',datadir,'.']);

%--------------------------------------------------------------------------
% TRIM FILE LISTS TO ONLY THOSE THAT HAVE WATERSHED CHARACTERISTICS
temp = hydro_files; hydro_files = {};
skip_hydro = {};
for i = 1:length(temp),
    if ~isempty(intersect(get_staid(temp{i}),(ws_char(:,1)))) % has ws_chars.
        hydro_files = [hydro_files;temp{i}];
    else
        skip_hydro = [skip_hydro;temp{i}];
    end
end
temp = skip_hydro; skip_hydro = [];
for i = 1:length(temp),
    skip_hydro = [skip_hydro;get_staid(temp{i})]; % extract staids.
end
skip_hydro = unique(skip_hydro);
skip_hydro(isnan(skip_hydro)) = [];

temp = hyeto_files; hyeto_files = {};
skip_hyeto = {};
for i = 1:length(temp),
    if ~isempty(intersect(get_staid(temp{i}),(ws_char(:,1)))) % has ws_chars.
        hyeto_files = [hyeto_files;temp{i}];
    else
        skip_hyeto = [skip_hyeto;temp{i}];
    end
end
temp = skip_hyeto; skip_hyeto = [];
for i = 1:length(temp),
    skip_hyeto = [skip_hyeto;get_staid(temp{i})]; % extract staids.
end
skip_hyeto = unique(skip_hyeto);
skip_hyeto(isnan(skip_hyeto)) = [];

n_skip = length(skip_hyeto);
if length(hydro_files) ~= length(hyeto_files),
    error('Hydro/hyeto file mismatch.');
elseif n_skip > 0,
    disp([num2str(n_skip),' stations have no watershed characteristics defined:']);

    %% one column output...
    %%       xxxxxxxx xxxxxxxx
    %disp('  hydro    hyeto');
    %for i = 1:length(skip_hyeto)
    %    disp(['  ',zeropad(skip_hydro(i),8),' ',zeropad(skip_hyeto(i),8)]);
    %end

    % 3 column output...
    %       xxxxxxxx/xxxxxxxx    xxxxxxxx/xxxxxxxx    xxxxxxxx/xxxxxxxx
    disp('  hydro    hyeto       hydro    hyeto       hydro    hyeto   ');
    left_over = mod(n_skip,3);
    for i = 1:3:n_skip-left_over,
        disp(['  ',zeropad(skip_hydro(i),8),  '/',zeropad(skip_hyeto(i),8),  '    ',...
                   zeropad(skip_hydro(i+1),8),'/',zeropad(skip_hyeto(i+1),8),'    ',...
                   zeropad(skip_hydro(i+2),8),'/',zeropad(skip_hyeto(i+2),8)]);
    end
    % display odd remaining...
    if left_over == 2,
        disp(['  ',zeropad(skip_hydro(end-1),8),'/',zeropad(skip_hyeto(end-1),8),'    ',...
                   zeropad(skip_hydro(end),8),  '/',zeropad(skip_hyeto(end),8)]);
    elseif left_over == 1,
        disp(['  ',zeropad(skip_hydro(end),8),  '/',zeropad(skip_hyeto(end),8)]);
    end
    
    disp('All files involving the above will be skipped.');
    disp(' ');
end

%--------------------------------------------------------------------------
% SORT BOTH ARRAYS ACCORDING TO THE HYDRO FILES IN DOWN-STREAM ORDER...

% create cell array of file names only...
fnames = [];
for i = 1:length(hydro_files),
    [pathstr, fname, ext, versn] = fileparts(hydro_files{i});
    fnames = [fnames;{fname}]; % add to list.
end
% sort by the file name - this puts in down-stream order.
[fnames,index] = sort(fnames);
hydro_files = hydro_files(index);
hyeto_files = hyeto_files(index);

%--------------------------------------------------------------------------
% CHECK FILE PAIRING OF FINAL OUTPUT
disp('* Checking hydro and hyeto file pairs for consistency ...');
if size(hydro_files) ~= size(hyeto_files),
    hydro_files = [];
    hyeto_files = [];
    error('File pairing check failed - list sizes are different.');
else
    for i = 1:length(hydro_files),
        err = check_pair(hydro_files{i},hyeto_files{i});
        if err,
            disp(['FOUND NON-MATCHING PAIR: ',hydro_files{i},' and ',hyeto_files{i}]);
            hydro_files = [];
            hyeto_files = [];
            error('File pairing check failed - a pair does not match up.');
        end
    end
end
disp('* File pairing is consistent');


% save as binary MAT file for 'use_last_fnames' option...
save('last_fnames.mat','hydro_files','hyeto_files'); 

% DONE.
return;


%==========================================================================
% AUXILARY FUNCTIONS

function err = check_pair(file1,file2)
err = 0;

% parse full path and file name into parts...
[path1,name1,ext1,ver1] = fileparts(file1);
[path2,name2,ext2,ver2] = fileparts(file2);

% check extensions...
if ~strcmp(ext1,ext2), % different extensions.
    err = 1;
    return;
end

% check that both come from same main directory...
slash = findstr(path1,'\');
dir1 = path1(1:slash(end)-1);
slash = findstr(path2,'\');
dir2 = path2(1:slash(end)-1);
if ~strcmp(dir1,dir2), % are in different main directories.
    err = 1;
    return;
end

% check that file names are same...
if ~strcmp(name1(5:end),name2(5:end)), % have different names.
    err = 1;
    return;
end

return;


