function [ws_char] = get_ws_char(charfile,USGS_DEM_AREA)
% USAGE: [STAIDS,SLOPES,LENGTHS,DEVELS,AREAS] = get_ws_parms('charfile',['USGS_DEM_AREA'])
%
% DESCRIPTION:
%   This function reads and returns the following watershed characteristics
%   from a CSV file.
%
% INPUT:
%   'charfile' - string containing the name of the input csv file.
%                It must be formated as:
%        STAID,   USGS_AREA[mi^2], DEM_AREA[mi^2], LENGTH[mi], SLOPE[mi/mi], DEVEL[0=undev;1=devel]
%        8042650, 6.82,            6.5617,         4.6323,     0.013778352,  0
%        8042700, 21.6,            23.9919,        11.5674,    0.006025265,  0
%        8048520, 17.7,            17.6345,        7.5298,     0.005080833,  1
%        ... more lines as necessary ...
%
%  ['USGS_DEM_AREA'] - (optional) string, either 'USGS' or 'DEM' indicating
%                      which drainage area value to ouput. If omitted, USGS
%                      area will be used.
%
% OUTPUT:
%   ws_char - numerical array with the following column vectors:
%     (col1) STAIDS  - USGS site identifier as an integer.
%     (col2) AREA    - drainage area      [mi^2] 
%     (col3) LENGTHS - channel length     [mi]
%     (col4) SLOPES  - channel slope      [dimensionless]
%     (col5) DEVEL   - development status [0=undeveloped, 1=developed]
%
% Written by Joe Vrabel, USGS May2006.

usage = 'USAGE: [SLOPES,LENGTHS,DEVELS,AREAS] = get_ws_parms(''charfile'',[''USGS_DEM_AREA''])';

% check input args...
if nargin < 1,
    disp(usage);
    error('Not enough input arguments.');
elseif nargin == 1, % need to default USGS_DEM_AREA.
    USGS_DEM_AREA = 'USGS';
elseif nargin > 2,
    disp(usage);
    error('Too many input arguments.');
end
if ~ischar(charfile),
    disp(usage);
    error('Input argument ''charfile'' must be a string.');
end
if ~ischar(USGS_DEM_AREA),
    disp(usage);
    error('Input argument ''USGS_DEM_AREA'' must be a string set to either ''USGS'' or ''DEM''.');
end
if exist(charfile,'file') ~= 2,
    disp(usage);
    error(['Input file ''',charfile,''' does not exist.']);
end

% read data file...
try
    data = csvread(charfile, 1, 0); % skip 1 header line.
catch
    disp(usage);
    error(['Cannot read input file: ''',charfile,'''.']);
end

% assign outputs...
%    1          2               3              4            5                 6
% [STAID, USGS_AREA[mi^2], DEM_AREA[mi^2], LENGTH[mi], SLOPE[mi/mi], DEVEL[0=undev;1=devel]]
STAIDS = data(:,1);
switch upper(USGS_DEM_AREA),
    case 'USGS', % use USGS area value.
        AREAS = data(:,2);
    case 'DEM', % use DEM area value.
        AREAS = data(:,3);
    otherwise
        disp(usage);
        error(['Unrecognaized value for input argument ''USGS_DEM_AREA'': ',USGS_DEM_AREA]);
end
LENGTHS = data(:,4);
SLOPES = data(:,5);
DEVELS = data(:,6);
ws_char = [STAIDS,AREAS,LENGTHS,SLOPES,DEVELS];
sortrows(ws_char,1); % sortby STAID.
if ~isequal(unique(ws_char(:,1)),ws_char(:,1)), % duplicate staid in list.
    error('Input data has multiple characteristic definitions for one or more stations.');
end

return;

