function [Hin] = parse_input(varargin)
% USAGE: [Hin] = parse_input(varargin)
%
% DESCRIPTION: this function parses multiple input arguments into
%   'batch_unit' and assigns the values into the data structure 'H.in'.
%   Arguments must be listed in 'PropertyName',property_value pairs, such
%   as: 'save_pdf',1. 'PropertyName' is case insensitive.
%
%   All specified input values are also checked for validity.
%
%   Parameters that are not specified in the 'PropertyName', property_value
%   pairs are assigned defaults.
%
% INPUT: varargin - cell array containing a variable list of input arguments.
%
% OUTPUT:
%   H.out - data structure with input values and defaults assigned.
%
% Written by: Joe Vrabel, USGS June2006

usage = 'USAGE: [H.in] = parse_input( ''PropertyName'',property_value, ... )';

% check input...
args = varargin{:};
if mod(length(args),2), % odd number of arguments - unpaired list.
    disp(usage);
    error('Input argument list ''varargin'' must be paired as: ''PropertyName'',property_value.');
end

% split PropertyNames and proerty_values...
names  = args(1:2:end);
values = args(2:2:end);

%--------------------------------------------------------------------------
% fill Hin data structure with default values...

% files and directories:

% create default name suffix: 'YYYYMMDD_HH.MM.SS' (orders alphabetically)
d = datevec(now);
suffix = [num2str(d(1)),zeropad(d(2),2),zeropad(d(3),2),'_',zeropad(d(4),2),'.',zeropad(d(5),2),'.',zeropad(round(d(6)),2)];

Hin.datadir = pwd;
Hin.use_last_fnames = 0; % pull fresh data file name list from 'datadir'.
Hin.outdir = ['.\OUTPUT_',suffix,'\']; % name of directory to put all output.
Hin.outfile = ['BATCH_UNIT_OUT_',suffix,'.csv']; % name of the output file.
Hin.logfile = ['BATCH_UNIT_LOG_',suffix,'.txt']; % name of run log file.
Hin.charfile = 'ws_char.csv'; % file containing watershed parms.

% precip parms:
Hin.use_synthP = 0; % use raw rain data.
Hin.P_trim1 = 0.01; % fraction of total interpolated precip volume to trim from start of precip.
Hin.P_trim2 = 0.01; % fraction of total interpolated precip volume to trim from end of precip.
Hin.no_neg_P = 1; % do not allow negative precip values.
Hin.no_neg_loss = 1; %do not allow negative loss function values.

% optimization parms:
Hin.Lfun = 'L1'; % 1-parm linear.
Hin.Lstr = 'L(t) = c_1'; % string for plotting.
Hin.obj = 1; % volume match.
Hin.obj_fun = @(estVtot,truVtot, estV,truV, estQ,truQ) abs(truVtot - estVtot); % actual function.
Hin.obj_str = '\Phi = |total VOL_t_r_u_e - total VOL_e_s_t|'; % string label for plotting.
Hin.opt_meth = 'fbnd'; % modified MatLab 'fminsearch', bounds opt parms found.
Hin.c0 = 0.000123; % initial c.
Hin.min_c = 0; % lower bound of c interval when monte carlo method used, and fmin/grid seach bounds.
Hin.max_c = 5; % upper bound of c interval when monte carlo method used, and fmin/grid seach bounds.
Hin.fmin_max_its = 1e10; % max number of fminsearch iterations.
Hin.fmin_obj_tol = 1e-6; % fminsearch obj func tolerance.
Hin.max_monte = 20; % maximum number of monte carlo iterations when stationary point encountered.
Hin.n_c1 = [20  20   50  50   100]; % grid search resolution in the c1 direction for successive passes.
Hin.n_c2 = [20  20   50  50   100]; % grid search resolution in the c2 direction for successive passes.
Hin.grid_tols = [0.5 0.25 0.1 0.05 0.01]; % relative sim/obs volume match tolerances to use fo rsuccessive grid search passes.

% misc:
Hin.save_pdf = 1; % save pdf's.
Hin.start_stop = [1,inf]; % run entire batch.
Hin.USGS_DEM_AREA = 'USGS'; % use USGS watershed areas.
Hin.max_array_size_MB = 750; % max size of array passed to calc_Q in MB - to avoid memory problems.
Hin.fracQp = 0.01; % fraction that specifies how long to run model:
                   % model terminates when gamma unit hydrograph decays to
                   % fracQp of Tp.
Hin.sim_obs_trim_pad_meth = 'pad_to_longer'; % pad the Q/V data to the longer one.
Hin.db_obj = 0; % do not create debug plot of objective function space.

%--------------------------------------------------------------------------
% over-write defaults with those values specified and check for validity...
user_outdir  = 0; % flag that shows if user specified an outdir.
user_outfile = 0; % flag that shows if user specified an outfile.
user_logfile = 0; % flag that shows if user specified an logfile.
user_c0 = 0; % flag saying if user specified c0.
for i = 1:length(names),
    if ~ischar(names{i}),
        disp(usage);
        error(['Input PropertyValue ',num2str(names{i}),' must be a string.']);
    end
    switch lower(names{i}),
        % DATADIR
        case 'datadir'
            Hin.datadir = values{i}; % assign.
            if ~ischar(Hin.datadir), % check.
                disp(usage);
                error('Input ''datadir'' must be a string.');
            elseif ~isdir(Hin.datadir),
                disp(usage);
                error('Input ''datadir'' must be an existing directory.');
            end
            
        % USE_LAST_FNAMES
        case 'use_last_fnames'
            Hin.use_last_fnames = values{i}; % assign.
            if ~isnumeric(Hin.use_last_fnames), % check.
                disp(usage);
                error('Input argument ''use_last_fnames'' must be either 1 or 0.');
            elseif ~isscalar(Hin.use_last_fnames),
                disp(usage);
                error('Input argument ''use_last_fnames'' must be either 1 or 0.');
            end
            Hin.use_last_fnames = round(Hin.use_last_fnames);
            if (Hin.use_last_fnames ~= 1) && (Hin.use_last_fnames ~= 0),
                disp(usage);
                error('Input argument ''use_last_fnames'' must be either 1 or 0.');
            end
        
        % OUTDIR
        case 'outdir'
            Hin.outdir = values{i}; % assign.
            if ~ischar(Hin.outdir), % check.
                disp(usage);
                error('Input ''outdir'' must be a string.');
            end
            user_outdir = 1; % user supplied an outdir.
            
        % OUTFILE
        case 'outfile'
            Hin.outfile = values{i}; % assign.
            if ~ischar(Hin.outfile), % check.
                disp(usage);
                error('Input ''outfile'' must be a string.');
            end
            user_outfile = 1; % user supplied an outfile.
            
        % LOGFILE
        case 'logfile'
            Hin.logfile = values{i}; % assign.
            if ~ischar(Hin.logfile), % check.
                disp(usage);
                error('Input ''logfile'' must be a string.');
            end
            user_logfile = 1; % user supplied an logfile.
        
        % CHARFILE
        case 'charfile'
            Hin.charfile = values{i}; % assign.
            if ~ischar(Hin.charfile), % check.
                disp(usage);
                error('Input ''charfile'' must be a string.');
            elseif exist(Hin.charfile,'file') ~= 2,
                disp(usage);
                error('Input ''charfile'' must be an existing file.');
            end    
        
        % USE_SYNTHP
        case 'use_synthp'
            Hin.use_synthP = values{i}; % assign.
            if ~isnumeric(Hin.use_synthP), % check.
                disp(usage);
                error('Input argument ''use_synthP'' must be either 1 or 0.');
            elseif ~isscalar(Hin.use_synthP),
                disp(usage);
                error('Input argument ''use_synthP'' must be either 1 or 0.');
            end
            Hin.use_synthP = round(Hin.use_synthP);
            if (Hin.use_synthP ~= 1) && (Hin.use_synthP ~= 0),
                disp(usage);
                error('Input argument ''use_synthP'' must be either 1 or 0.');
            end
            
        % P_TRIM1
        case 'p_trim1'
            Hin.P_trim1 = values{i}; % assign.
            if ~isnumeric(Hin.P_trim1), % check.
                disp(usage);
                error('Input argument ''P_trim1'' must be a real number between 0 and 1 inclusive.');
            elseif ~isscalar(Hin.P_trim1),
                disp(usage);
                error('Input argument ''P_trim1'' must be a real number between 0 and 1 inclusive.');
            elseif (Hin.P_trim1 < 0) || (Hin.P_trim1 > 1),
                disp(usage);
                error('Input argument ''P_trim1'' must be a real number between 0 and 1 inclusive.');
            end
            
        % P_TRIM2
        case 'p_trim2'
            Hin.P_trim2 = values{i}; % assign.
            if ~isnumeric(Hin.P_trim2), % check.
                disp(usage);
                error('Input argument ''P_trim2'' must be a real number between 0 and 1 inclusive.');
            elseif ~isscalar(Hin.P_trim2),
                disp(usage);
                error('Input argument ''P_trim2'' must be a real number between 0 and 1 inclusive.');
            elseif (Hin.P_trim2 < 0) || (Hin.P_trim2 > 1),
                disp(usage);
                error('Input argument ''P_trim2'' must be a real number between 0 and 1 inclusive.');
            end
            
        % NO_NEG_P
        case 'no_neg_p'
            Hin.no_neg_P = values{i}; % assign.
            if ~isnumeric(Hin.no_neg_P), % check.
                disp(usage);
                error('Input argument ''no_neg_P'' must be either 1 or 0.');
            elseif ~isscalar(Hin.no_neg_P),
                disp(usage);
                error('Input argument ''no_neg_P'' must be either 1 or 0.');
            end
            Hin.no_neg_P = round(Hin.no_neg_P);
            if (Hin.no_neg_P ~= 1) && (Hin.no_neg_P ~= 0),
                disp(usage);
                error('Input argument ''no_neg_P'' must be either 1 or 0.');
            end
            
        % NO_NEG_LOSS
        case 'no_neg_loss'
            Hin.no_neg_loss = values{i}; % assign.
            if ~isnumeric(Hin.no_neg_loss), % check.
                disp(usage);
                error('Input argument ''no_neg_loss'' must be either 1 or 0.');
            elseif ~isscalar(Hin.no_neg_loss),
                disp(usage);
                error('Input argument ''no_neg_loss'' must be either 1 or 0.');
            end
            Hin.no_neg_loss = round(Hin.no_neg_loss);
            if (Hin.no_neg_loss ~= 1) && (Hin.no_neg_loss ~= 0),
                disp(usage);
                error('Input argument ''no_neg_loss'' must be either 1 or 0.');
            end
            
        % LFUN
        case 'lfun'
            Hin.Lfun = values{i}; % assign.
            if ~ischar(Hin.Lfun), % check.
                disp(usage);
                error('Input argument ''Lfun'' must be a string.');
            else
                Hin.Lfun = upper(Hin.Lfun);
            end
            % assign loss function string...
            switch upper(Hin.Lfun),
                case 'L1', % 1-parameter linear loss.
                    Hin.Lstr = 'L(t) = c_1';
                case 'L2',
                    Hin.Lstr = 'L(t) = c_1t + c_2';
                case 'L3',
                    Hin.Lstr = 'L(t) = c_1t^2 + c_2t + c_3';
                case 'E1',
                    Hin.Lstr = 'L(t) = exp(-c_1t)';
                case 'E2',
                    Hin.Lstr = 'L(t) = c_1exp(-c2t)';
                case 'E3',
                    Hin.Lstr = 'L(t) = c_1exp( -(c_2t + c_3))';
                case 'E4',
                    Hin.Lstr = 'L(t) = c_1exp( -(c_2t + c_3)) + c_4';
                case 'P1',
                    Hin.Lstr = 'P_x_s(t) = c_1P_t_o_t(t) [0<=c_1<= 1]';
                case 'A1',
                    Hin.Lstr = 'P_x_s(t) = P_t_o_t - init.abs(c_1 P_t_o_t) [0<=c_1<= 1]';
                case 'AC2',                    
                    Hin.Lstr = 'P_x_s(t) = init.abs. then const.loss';
                case 'AP2',
                    Hin.Lstr = 'P_x_s(t) = init.abs. then prop.loss';
                otherwise,
                    disp(usage);
                    error(['Unrecognized loss function: ',Hin.Lfun]);
            end
            
        % OBJ
        case 'obj'
            Hin.obj = values{i}; % assign.
            if ~isnumeric(Hin.obj), % check.
                disp(usage);
                error('Input argument ''obj'' must be an integer scalar.');
            elseif ~isscalar(Hin.obj),
                disp(usage);
                error('Input argument ''obj'' must be an integer scalar.');
            end
            Hin.obj = round(Hin.obj);
            % set objective function using the following variables:
            %      estVtot - total estimated volume.
            %      truVtot - total actual volume.
            %      estV - estimated volume time-series.
            %      truV - actual volume time-series.
            %      estQ - estimated discharge time-series.
            %      truQ - actual discharge time-series.
            Hin.obj_fun = [];
            Hin.obj_str = [];
            if Hin.obj == 1, % minimize total volume residual.
                Hin.obj_fun = @(estVtot,truVtot, estV,truV, estQ,truQ) abs(truVtot - estVtot);
                Hin.obj_str = '\Phi = |total VOL_t_r_u_e - total VOL_e_s_t|'; % string label for plotting.
            elseif Hin.obj == 2, % minimize continuous flow residuals.
                Hin.obj_fun = @(estVtot,truVtot, estV,truV, estQ,truQ) sum(abs(truQ - estQ));
                Hin.obj_str = '\Phi = \Sigma|Q_t_r_u_e - Q_e_s_t|'; % string label for plotting.
            elseif Hin.obj == 3, % minimize continuous volume residuals.
                Hin.obj_fun = @(estVtot,truVtot, estV,truV, estQ,truQ) sum(abs(truV - estV));
                Hin.obj_str = '\Phi = \Sigma|VOL_t_r_u_e - VOL_e_s_t|'; % string label for plotting.
            elseif Hin.obj == 4, % continuous flow least-squares.
                Hin.obj_fun = @(estVtot,truVtot, estV,truV, estQ,truQ) sum((truQ - estQ).^2);
                Hin.obj_str = '\Phi = \Sigma(Q_t_r_u_e - Q_e_s_t)^2'; % string label for plotting.
            elseif Hin.obj == 5, % continuous volume least-squares.
                Hin.obj_fun = @(estVtot,truVtot, estV,truV, estQ,truQ) sum((truV - estV).^2);
                Hin.obj_str = '\Phi = \Sigma(VOL_t_r_u_e - VOL_e_s_t)^2'; % string label for plotting.
            else
                disp(usage);
                error('Parameter ''obj'' must be 1,2,3,4, or 5.');
            end
            if isempty(Hin.obj_fun) || isempty(Hin.obj_str), % one or both didn't get assigned - error.
                disp(usage);
                error('Model parameter ''obj'' not set correctly.');
            end            
        
        % OPT_METH
        case 'opt_meth'
            Hin.opt_meth = lower(values{i}); % assign.
            if ~strcmp(Hin.opt_meth,'fmin') && ...
               ~strcmp(Hin.opt_meth,'fbnd') && ...
               ~strcmp(Hin.opt_meth,'grid_search') && ...
               ~strcmp(Hin.opt_meth,'none'),
                disp(usage);
                error('Input argument ''opt_meth'' must be either ''fmin'', ''fbnd'', ''grid_search'', or ''none''.');
            end
        
        % C0
        case 'c0'
            Hin.c0 = values{i}; % assign.
            %% can have a character string value specifying a file name of c0 values.
            %if ~isnumeric(Hin.c0), % check.
            %    disp(usage);
            %    error('Input argument ''c0'' must be numeric.');
            %end
            user_c0 = 1; % flag saying user specified c0.
            
        % MIN_C
        case 'min_c'
            Hin.min_c = values{i}; % assign.
            if ~isnumeric(Hin.min_c), % check.
                disp(usage);
                error('Input argument ''min_c'' must be numeric scalar or vector.');
            elseif ~isvector(Hin.min_c),
                disp(usage);
                error('Input argument ''min_c'' must be numeric scalar or vector.');
            end
            
        % MAX_C
        case 'max_c'
            Hin.max_c = values{i}; % assign.
            if ~isnumeric(Hin.max_c), % check.
                disp(usage);
                error('Input argument ''max_c'' must be numeric scalar or vector.');
            elseif ~isvector(Hin.max_c),
                disp(usage);
                error('Input argument ''max_c'' must be numeric scalar or vector.');
            end

        % FMIN_MAX_ITS
        case 'fmin_max_its'
            Hin.fmin_max_its = values{i}; % assign.
            if ~isnumeric(Hin.fmin_max_its), % check.
                disp(usage);
                error('Input argument ''fmin_max_its'' must be an integer scalar greater than 0.');
            end
            if ~isscalar(Hin.fmin_max_its),
                disp(usage);
                error('Input argument ''fmin_max_its'' must be an integer scalar greater than 0.');
            end
            Hin.fmin_max_its = ceil(Hin.fmin_max_its); % round up.
            if Hin.fmin_max_its <= 0,
                disp(usage);
                error('Input argument ''fmin_max_its'' must be an integer scalar greater than 0.');
            end
            
        % FMIN_OBJ_TOL
        case 'fmin_obj_tol'
            Hin.fmin_obj_tol = values{i}; % assign.
            if ~isnumeric(Hin.fmin_obj_tol), % check.
                disp(usage);
                error('Input argument ''fmin_obj_tol'' must be a numeric scalar greater than 0.');
            end
            if ~isscalar(Hin.fmin_obj_tol),
                disp(usage);
                error('Input argument ''fmin_obj_tol'' must be a numeric scalar greater than 0.');
            end
            if Hin.fmin_obj_tol <= 0,
                disp(usage);
                error('Input argument ''fmin_obj_tol'' must be a numeric scalar greater than 0.');
            end     
            
        % MAX_MONTE
        case 'max_monte'
            Hin.max_monte = values{i}; % assign.
            if ~isnumeric(Hin.max_monte), % check.
                disp(usage);
                error('Input argument ''max_monte'' must be an interger greater than 0.');
            elseif ~isscalar(Hin.max_monte),
                disp(usage);
                error('Input argument ''max_monte'' must be an interger greater than 0.');
            end
            Hin.max_monte = round(Hin.max_monte);
            if Hin.max_monte <= 0,
                disp(usage);
                error('Input argument ''max_monte'' must be an interger greater than 0.');
            end
            
        % N_C1
        case 'n_c1'
            Hin.n_c1 = values{i}; % assign.
            if ~isnumeric(Hin.n_c1), % check.
                disp(usage);
                error('Input argument ''n_c1'' must be an integer vector with all elements greater than 2.');
            end
            if ~isvector(Hin.n_c1),
                disp(usage);
                error('Input argument ''n_c1'' must be an integer vector with all elements greater than 2.');
            end
            Hin.n_c1 = ceil(Hin.n_c1); % round up.
            if sum(Hin.n_c1>2) ~= length(Hin.n_c1), % at least 1 element <= 2.
                disp(usage);
                error('Input argument ''n_c1'' must be an integer vector with all elements greater than 2.');
            end
            
        % N_C2
        case 'n_c2'
            Hin.n_c2 = values{i}; % assign.
            if ~isnumeric(Hin.n_c2), % check.
                disp(usage);
                error('Input argument ''n_c2'' must be an integer vector with all elements greater than 2.');
            end
            if ~isvector(Hin.n_c2),
                disp(usage);
                error('Input argument ''n_c2'' must be an integer vector with all elements greater than 2.');
            end
            Hin.n_c2 = ceil(Hin.n_c2); % round up.
            if sum(Hin.n_c2>2) ~= length(Hin.n_c2), % at least 1 element <= 2.
                disp(usage);
                error('Input argument ''n_c2'' must be an integer vector with all elements greater than 2.');
            end
        
        % GRID_TOLS
        case 'grid_tols'
            Hin.grid_tols = values{i}; % assign.
            if ~isnumeric(Hin.grid_tols), % check.
                disp(usage);
                error('Input argument ''grid_tols'' must be a numerical vector with all elements greater than 0.');
            end
            if ~isvector(Hin.grid_tols),
                disp(usage);
                error('Input argument ''grid_tols'' must be a numerical vector with all elements greater than 0.');
            end
            if sum(Hin.grid_tols>0) ~= length(Hin.grid_tols), % at least 1 element <= 0.
                disp(usage);
                error('Input argument ''grid_tols'' must be a numerical vector with all elements greater than 0.');
            end
            
        % SAVE_PDF         
        case 'save_pdf'
            Hin.save_pdf = values{i}; % assign.
            if ~isnumeric(Hin.save_pdf), % check.
                disp(usage);
                error('Input argument ''save_pdf'' must be either 1 or 0.');
            elseif ~isscalar(Hin.save_pdf),
                disp(usage);
                error('Input argument ''save_pdf'' must be either 1 or 0.');
            elseif (Hin.save_pdf ~= 1) && (Hin.save_pdf ~= 0),
                disp(usage);
                error('Input argument ''save_pdf'' must be either 1 or 0.');
            end

        % START_STOP
        case 'start_stop'
            Hin.start_stop = values{i}; % assign.
            if ~isnumeric(Hin.start_stop) || sum(size(Hin.start_stop)==[1,2]) ~= 2, % check.
                disp(usage);
                error('Input argument ''start_stop'' must be a positive integer vector of form [start,stop], with start <= stop.');
            end
            Hin.start_stop = round(Hin.start_stop);
            if Hin.start_stop(1) <= 0 || Hin.start_stop(2) <= 0 || Hin.start_stop(1) > Hin.start_stop(2),
                disp(usage);
                error('Input argument ''start_stop'' must be a positive integer vector of form [start,stop], with start <= stop.');
            end

        % USGS_DEM_AREA
        case 'usgs_dem_area'
            Hin.USGS_DEM_AREA = upper(values{i}); % assign.
            if ~strcmp(Hin.USGS_DEM_AREA,'USGS') && ~strcmp(Hin.USGS_DEM_AREA,'DEM'), % check.
                disp(usage);
                error('Input ''USGS_DEM_AREA'' must be either ''USGS'' or ''DEM''.');
            end
            
        % MAX_ARRAY_SIZE_MB
        case 'max_array_size_mb'
            Hin.max_array_size_MB = values{i}; % assign.
            if ~isnumeric(Hin.max_array_size_MB), % check.
                disp(usage);
                error('Input ''max_array_size_MB'' must be a positive real number.');
            elseif ~isscalar(Hin.max_array_size_MB),
                disp(usage);
                error('Input ''max_array_size_MB'' must be a real number greater than 0.');
            elseif Hin.max_array_size_MB <= 0,
                disp(usage);
                error('Input ''max_array_size_MB'' must be a real number greater than 0.');
            end
            
        % FRACTP
        case 'fracqp'
            Hin.fracQp = values{i}; % assign.
            if ~isnumeric(Hin.fracQp), % check.
                disp(usage);
                error('Input ''fracQp'' must be a real number between 0 and 1 inclusive.');
            elseif ~isscalar(Hin.fracQp),
                disp(usage);
                error('Input ''fracQp'' must be a real number between 0 and 1 inclusive.');
            elseif (Hin.fracQp < 0) || (Hin.fracQp > 1),
                disp(usage);
                error('Input ''fracQp'' must be a real number between 0 and 1 inclusive.');
            end
        
        % SIM_OBS_TRIM_PAD_METH
        case 'sim_obs_trim_pad_meth'
            Hin.sim_obs_trim_pad_meth = lower(values{i}); % assign.
            if ~strcmp(Hin.sim_obs_trim_pad_meth,'trim_pad_to_obs') && ...
               ~strcmp(Hin.sim_obs_trim_pad_meth,'trim_pad_to_sim') && ...
               ~strcmp(Hin.sim_obs_trim_pad_meth,'trim_to_shorter') && ...
               ~strcmp(Hin.sim_obs_trim_pad_meth,'pad_to_longer'), % invalid entry.
                disp(usage);
                error(['Input ''sim_obs_trim_pad_meth'' must be one of either ' ...
                       '''trim_pad_to_obs'', ' ...
                       '''trim_pad_to_sim'', ' ...
                       '''trim_to_shorter'', or ' ...
                       '''pad_to_longer''.']);
            end
            
        % DB_OBJ         
        case 'db_obj'
            Hin.db_obj = values{i}; % assign.
            if ~isnumeric(Hin.db_obj),
                disp(usage);
                error('Input argument ''db_obj'' must be either 1 or 0.');
            elseif ~isscalar(Hin.db_obj),
                disp(usage);
                error('Input argument ''db_obj'' must be either 1 or 0.');
            elseif (Hin.db_obj ~= 1) && (Hin.db_obj ~= 0), % check.
                disp(usage);
                error('Input argument ''db_obj'' must be either 1 or 0.');
            end
            
        otherwise
            disp(usage);
            error(['Unrecognized PropertyValue: ',names{i}]);
    end
end

%--------------------------------------------------------------------------
% MISCELANEOUS CONSISTENCY CHECKS...

% DON'T DO THIS - HAVE USER MAKE EVERYTHING CONSISTENT
% % if only 1 c0 value given for multiple parameter loss function, create
% % vector with correct size with duplicate values...
% if numel(Hin.c0) == 1,
%     Hin.c0 = Hin.c0*ones(1,str2double(Hin.Lfun(end))); % vector with same number of elements as loss parms.
% end

% if 'opt_meth' = 'none', make sure user set a 'c0'...
if strcmp(Hin.opt_meth,'none') && ~user_c0,
    disp(usage);
    error('The ''none'' optimization mode requires an explicit user-defined ''c0'' to be set.');
end

% if 'opt_meth' = 'grid_search', make sure loss function is 2-parameter...
if strcmp(Hin.opt_meth,'grid_search') && (Hin.Lfun(end) ~= '2'),
    disp(usage);
    error('The ''grid_search'' optimization method can only be used with a 2-parameter loss function.');
end

% check c0...
if isnumeric(Hin.c0), % numeric c0 value entered.
    % make sure c0 has correct number of elements required for specified loss function...
    if numel(Hin.c0) ~= str2double(Hin.Lfun(end)),
        disp(usage);
        error(['Loss function ''',Hin.Lfun,''' requires a ',Hin.Lfun(end),' element vector for c0.']);
    end

    % make sure c0 same size as min_c...
    if sum(size(Hin.c0) == size(Hin.min_c)) ~= 2, % different dimensions.
        disp(usage);
        error('Inputs ''c0'', ''min_c'' and ''max_c'' must all have the same size.');
    end
elseif ischar(Hin.c0), % string entered.
    if exist(Hin.c0,'file') ~= 2,
        disp(usage);
        error(['Input file ''',Hin.c0,''' does not exist.']);
    end
else
    disp(usage);
    error('Input ''c0'' must be numeric or a string.');
end

% make sure min_c same size as max_c...
if sum(size(Hin.min_c) == size(Hin.max_c)) ~= 2, % different dimensions.
    disp(usage);
    error('Inputs ''min_c'' and ''max_c'' must have the same size.');
end

% make sure min_c < max_c...
if sum(Hin.min_c >= Hin.max_c) > 0, % at least 1 element of min_c is >= to corresponding element of max_c.
    disp(usage);
    error('Input ''max_c'' must be element-wise greater than ''min_c''.');
end

% make sure n_c1 same size as n_c2...
if sum(size(Hin.n_c1) == size(Hin.n_c2)) ~= 2, % different dimensions.
    disp(usage);
    error('Inputs ''n_c1'', ''n_c2'', and ''grid_tols'' must have the same size.');
end

% make sure n_c1 same size as grid_tols...
if sum(size(Hin.n_c1) == size(Hin.grid_tols)) ~= 2, % different dimensions.
    disp(usage);
    error('Inputs ''n_c1'', ''n_c2'', and ''grid_tols'' must have the same size.');
end

% append opt_meth to obj_str...
Hin.obj_str = [Hin.obj_str,' (',Hin.opt_meth(1:4),')'];

% append opt_meth, loss function, and objective function to outdir IF user did not specify his own outdir...
if ~user_outdir,
    if Hin.outdir(end) == '\', % get rid of any trailing '\'.
        Hin.outdir = Hin.outdir(1:end-1);
    end
    Hin.outdir = [Hin.outdir,'_',lower(Hin.opt_meth(1:4)),'.',Hin.Lfun,'.',num2str(Hin.obj),'\'];
end

% append opt_meth, loss function, and objective function to outfile IF user did not specify his own outfile...
if ~user_outfile,
    [pathstr, name, ext] = fileparts(Hin.outfile); % get file parts.
    Hin.outfile = [name,'_',lower(Hin.opt_meth(1:4)),'.',Hin.Lfun,'.',num2str(Hin.obj),ext];
end

% append opt_meth, loss function, and objective function to logfile IF user did not specify his own logfile...
if ~user_logfile,
    [pathstr, name, ext] = fileparts(Hin.logfile); % get file parts.
    Hin.logfile = [name,'_',lower(Hin.opt_meth(1:4)),'.',Hin.Lfun,'.',num2str(Hin.obj),ext];
end

% build full outfile and logfile names...
Hin.outfile = [Hin.outdir,Hin.outfile];
Hin.logfile = [Hin.outdir,Hin.logfile];

return;

