function [] = plot_obj(Hin)
% USAGE: plot_obj(Hin)
%
% DESCRIPTION: This function plots the objective function space of an
%   "optimized" model run along with the c_opt found. This is
%   computationally expensive and is not normally done, but can be very
%   useful for diagnosing a pathological run that is either not converging
%   or converges to a bad value. The batch run is terminated after this
%   function is complete, and the plot of the objective function is left
%   on-screen.
%
%   The objective space is bounded by run parameters min_c and max_c.
%
%   Only 1- and 2-parameter loss function models are supported.
%
% INPUT:
%   Hin - data structure containing run parameters and output.
%
% OUTPUT:
%   1- or 2-dimensional objective function plot.
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: plot_obj(Hin)';

% check input args...
if nargin ~= 1,
    disp(usage);
    error('Incorrect number of input arguments.');
end
if ~isstruct(Hin),
    disp(usage);
    error('Input argument ''Hin'' must be a data structure.');
end

% turn off flat-line checking...
Hin.check_flat = 0; % do not check for flat-line models.
Hin.OBJ = @(c) calc_obj(Hin,c); % update OBJ function.

%--------------------------------------------------------------------------
scr = get(0,'ScreenSize'); % [left bottom width height]
figure('Position',[scr(1),scr(2)+0.01,scr(3),0.94*scr(4)], ...
       'NumberTitle','off','Name','DEBUG: objective function space');
if length(Hin.c0) == 1, % single parameter loss function.
    nx = 100; % resolution of 2-D plot (calculates nx values).
    X = linspace(Hin.min_c,Hin.max_c,nx);
    Y = NaN(size(X));
    nx = 0;
    for i = X,
        nx = nx + 1;
        Y(nx) = Hin.OBJ(i);
    end
    % add actual min so it is in the objective function plot...
    % (otherwise if resolution too course it will appear to be off the line)
    [X,index] = sort([X,Hin.c_opt]); % add c_opt to X.
    obj_c_opt = Hin.OBJ(Hin.c_opt); % objective function at c_opt.
    Y = [Y,obj_c_opt]; % add objective function at c_opt to Y.
    Y = Y(index); % put Y in correct order.
    
    % get rid of any realmax's - occur when model flat-lines at 0 volume.
    index = (Y==realmax);
    X(index) = [];
    Y(index) = [];

    % plot results...
    plot(X,Y); hold on; % objective curve.
    plot(Hin.c_opt,obj_c_opt,'ro'); % c_opt found.
    title(['\bfOBJ FUNCTION: ',Hin.obj_str,'      LOSS FUNCTION: ',Hin.Lstr]);
    xlabel('C_1 (UNADJUSTED)');
    ylabel('OBJECTIVE FUNCTION');
    legend('objective function','minimum found');
    
elseif length(Hin.c0) == 2, % 2-parameter loss function.
    nx = 20; % resolution of 3-D plot (calculates nx x ny values).
    ny = 20; % resolution of 3-D plot (calculates nx x ny values).
    [X,Y] = meshgrid(linspace(Hin.min_c(1),Hin.max_c(1),nx),linspace(Hin.min_c(2),Hin.max_c(2),ny));
    Z = NaN(size(X));
    for i = 1:length(X), % x-dir.
        for j = 1:length(Y), % y-dir.
            Z(i,j) = Hin.OBJ([Y(j,j),X(i,i)]);
        end
    end
    surf(X,Y,Z); colormap('jet'); hold on;
    plot3(Hin.c_opt(1),Hin.c_opt(2),Hin.OBJ([Hin.c_opt(1),Hin.c_opt(2)]),'ro'); % add actual c_opt found.
    title(['\bfOBJECTIVE FUNCTION MINIMIZATION: ',Hin.obj_str]);
    xlabel('C_1  (UNADJUSTED)');
    ylabel('C_2  (UNADJUSTED)');
    zlabel('OBJECTIVE FUNCTION');
%     legend('objective function','minimum found');
else
    error('db_opt only supports 1- and 2-parameter loss functions.');
end

% turn flat-line checking back on...
Hin.check_flat = 0; % do not check for flat-line models.
Hin.OBJ = @(c) calc_obj(Hin,c); % update OBJ function.

% close all files and stop batch...
fclose all;
diary off;
error('DEBUG STOP: plot_obj');

return;

