function [] = plot_out(Hin)
% USAGE: plot_out(Hin)
%
% DESCRIPTION: This function creates a plot for a 'batch_unit' run and
%   saves it as an individual PDF in the output directory.
%
% INPUT:
%   Hin - data structure containing needed model output and run data.
%
% OUTPUT:
%   PDF file of plot.
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: plot_out(Hin)';

% check input args...
if nargin ~= 1,
    disp(usage);
    error('Incorrect number of input arguments.');
end
if ~isstruct(Hin),
    disp(usage);
    error('Input argument ''Hin'' must be a data structure.');
end


% get staid-date and assign PDF name...
[pathstr, staid_date, ext, versn] = fileparts(Hin.hydro_file);
staid_date = staid_date(6:end); % get rid of leading 'unit_' characters and add run#.
pdf_name = ['run',zeropad(Hin.run_num,4),'_',staid_date]; % run number and staid-date: 'run####_########_YYYY_MMDD' (assumes max run_num = 9999)

scr = get(0,'ScreenSize'); % [left bottom width height]
figure('Position',[scr(1),scr(2)+0.01,scr(3),0.94*scr(4)], ...
       'Name','BATCH_UNIT: plot of results','NumberTitle','off'); % make full-screen and give title.

%--------------------------------------------------------------------------
% add text in upper left corner...
subplot(4,4,[1,6]);
axis off;
xpos = 0;
n_txt = 29;
ypos = linspace(1,0,n_txt);
text(xpos,ypos( 1),['STAID-DATE: ',staid_date],'FontSize',8,'Interpreter','none','FontName','courier');
text(xpos,ypos( 2),['DATA DIR: ',Hin.DATADIR],'FontSize',8,'Interpreter','none','FontName','courier');
text(xpos,ypos( 3),['AREA [mi^2] ............................... ',num2str(Hin.AREA)],'FontSize',8,'FontName','courier');
text(xpos,ypos( 4),'--------------------- PRECIPITATION -----------------------','FontSize',8,'FontName','courier');
text(xpos,ypos( 5),['TOTAL RAIN VOLUME [inches] ............... ',num2str(Hin.RAIN_TOT_VOL)],'FontSize',8,'FontName','courier');
text(xpos,ypos( 6),['EXCESS RAIN VOLUME [inches] .............. ',num2str(Hin.RAIN_XS_VOL)],'FontSize',8,'FontName','courier');
text(xpos,ypos( 7),['PECENT RAIN VOLUME LOSS .................. ',num2str(Hin.RAIN_PER_VOL_LOSS)],'FontSize',8,'FontName','courier');
text(xpos,ypos( 8),'----------------------- DISCHARGE -------------------------','FontSize',8,'FontName','courier');
text(xpos,ypos( 9),['MEAN OBS Q [CFS] ......................... ',num2str(Hin.OBS_Q_MEAN)],'FontSize',8,'FontName','courier');
text(xpos,ypos(10),['MEAN SIM Q [CFS] ......................... ',num2str(Hin.SIM_Q_MEAN)],'FontSize',8,'FontName','courier');
text(xpos,ypos(11),['RMS Q RESIDUALS [CFS] .................... ',num2str(Hin.RMS_Q_RESID)],'FontSize',8,'FontName','courier');
text(xpos,ypos(12),['Q RELATIVE BIAS .......................... ',num2str(Hin.Q_REL_BIAS)],'FontSize',8,'FontName','courier');
text(xpos,ypos(13),['Q NASH-SUTCLIFFE EFFICIENCY .............. ',num2str(Hin.Q_NS_EFF)],'FontSize',8,'FontName','courier');
text(xpos,ypos(14),['Q SIM vs OBS R^2 .......................... ',num2str(Hin.Q_SIM_OBS_R2)],'FontSize',8,'FontName','courier');
text(xpos,ypos(15),['Q SIM vs OBS SLOPE ....................... ',num2str(Hin.Q_SIM_OBS_SLOPE)],'FontSize',8,'FontName','courier');
text(xpos,ypos(16),['Q SIM vs OBS INTERCEPT ................... ',num2str(Hin.Q_SIM_OBS_INT)],'FontSize',8,'FontName','courier');
text(xpos,ypos(17),'------------------------- VOLUME --------------------------','FontSize',8,'FontName','courier');
text(xpos,ypos(18),['MEAN OBS V [CFS] ......................... ',num2str(Hin.OBS_V_MEAN)],'FontSize',8,'FontName','courier');
text(xpos,ypos(19),['MEAN SIM V [CFS] ......................... ',num2str(Hin.SIM_V_MEAN)],'FontSize',8,'FontName','courier');
text(xpos,ypos(20),['RMS V RESIDUALS [CFS] .................... ',num2str(Hin.RMS_V_RESID)],'FontSize',8,'FontName','courier');
text(xpos,ypos(21),['V RELATIVE BIAS .......................... ',num2str(Hin.V_REL_BIAS)],'FontSize',8,'FontName','courier');
text(xpos,ypos(22),['V NASH-SUTCLIFFE EFFICIENCY .............. ',num2str(Hin.V_NS_EFF)],'FontSize',8,'FontName','courier');
text(xpos,ypos(23),['V SIM vs OBS R^2 .......................... ',num2str(Hin.V_SIM_OBS_R2)],'FontSize',8,'FontName','courier');
text(xpos,ypos(24),['V SIM vs OBS SLOPE ....................... ',num2str(Hin.V_SIM_OBS_SLOPE)],'FontSize',8,'FontName','courier');
text(xpos,ypos(25),['V SIM vs OBS INTERCEPT ................... ',num2str(Hin.V_SIM_OBS_INT)],'FontSize',8,'FontName','courier');
text(xpos,ypos(26),'----------------- OPTIMIZATION RESULTS --------------------','FontSize',8,'FontName','courier');
text(xpos,ypos(27),['SIM/OBS TOTAL VOLUME RATIO ............... ',num2str(Hin.SIM_OBS_TOT_VOL_RATIO)],'FontSize',8,'FontName','courier');
text(xpos,ypos(28),['MINIMIZED OBJECTIVE FUNCTION VALUE ....... ',num2str(Hin.obj_opt)],'FontSize',8,'FontName','courier');
text(xpos,ypos(29),['C_o_p_t: ',num2str(Hin.c_opt_out)],'FontSize',8,'FontName','courier');
title(['\bf','RUN #',num2str(Hin.run_num)],'FontSize',8); % use run number as title.

%--------------------------------------------------------------------------
% precip plots

% get time vectors to use - need to match up precip and flow in time:
% set 1st rain pulse (event_start) to 0 hour, then adjust 1st flow time by this.
Train = 24*(0 : 1/288 : (length(Hin.RAIN_USE)-1)/(12*24))'; % [hours]
Tmod  = 24*(Hin.Q_RAW(1:1)-Hin.event_start  : 1/288 : (Hin.Q_RAW(1,1)-Hin.event_start) + (length(Hin.V_SIM)-1)/(12*24))'; % [hours]
x_range = [min(Train(1),Tmod(1)),max(Train(end),Tmod(end))]; % time range for all plots.
% if Hin.pad <= 0,
%     disp('pad a')
%     Hin.pad
% else
%     disp('pad b')
%     Hin.pad
% end

% plot loss function...
subplot(4,4,[3 4]); hold on;
% plot((5/60)*(0:length(Hin.loss)-1)', Hin.loss);
plot(Train, Hin.loss);
plot(get(gca,'Xlim'),[0 0],'k'); % y=0 line.
title(['\bf','PRECIP LOSS FUNCTION: ',Hin.Lstr],'FontSize',8);
xlabel(Hin.obj_str,'FontSize',8); % use objective function as x-label.
ylabel('LOSS RATE [INCH/5MIN]','FontSize',8);
ylims = get(gca,'Ylim');
axis([x_range,-inf,ylims(2) + 0.05*(ylims(2)-ylims(1))]); % give 10% buffer head room.
box on;

% plot rain series...
subplot(4,4,[7 8]); hold on;
% bar(Train,Hin.RAIN,     'EdgeColor',[0.0 0.0 1.0], 'FaceColor',[0.0 0.0 1.0]); % raw.
bar(Train,Hin.RAIN_USE, 'EdgeColor',[0.2 0.5 0.2], 'FaceColor',[0.2 0.5 0.2]); % total modeled.
bar(Train,Hin.RAIN_XS,  'EdgeColor',[1.0 0.0 0.0], 'FaceColor',[1.0 0.0 0.0]); % excess.
if Hin.use_synthP,
    title(['\bf','OBS AND MODELED RAINFALL: SYNTHETIC DATA USED'],'FontSize',8);
else
    title(['\bf','OBS AND MODELED RAINFALL: RAW DATA USED'],'FontSize',8);
end
xlabel('TIME [HOURS]','FontSize',8);
ylabel('PRECIP [INCHES]','FontSize',8);
% legend(['actual rainfall (total: ',num2str(sum(RAIN)),' inches)'], ...
%        ['total modeled rainfall (total: ',num2str(sum(Ptot)),' inches)'], ...
%        ['excess modeled rainfall (',num2str(100*(sum(Ptot)-sum(Pxs))/sum(Ptot), 3),'% loss)']);
axis([x_range,-inf,inf]);
box on;

%--------------------------------------------------------------------------
% plot volumes and residuals...
h_V_ax = subplot(4,4,[9 10]); hold on; % volume.
plot(Tmod,Hin.V_OBS,'Color',[0 0.5 0]); % measured.
plot(Tmod,Hin.V_SIM,'Color',[1 0.0 0]); % result from unit hydrograph.
title(['\bf','OBS AND ESTIMATED ACCUMULATED VOLUME'],'FontSize',8);
xlabel('TIME [HOURS]','FontSize',8);
ylabel('VOLUME [INCHES]','FontSize',8);
% legend(['actual volume (total ',num2str(V_OBS(end)),' inches)'], ...
%        ['estimated volume (total ',num2str(Vtot),' inches)'], ...
%         'Location','southeast');
axis([x_range,-inf,inf]);
box on;

subplot(4,4,[13 14]);  hold on; % residuals.
bar(Tmod,(Hin.V_OBS-Hin.V_SIM),'EdgeColor',[1 0 0], 'FaceColor',[1 0 0]);
title(['\bf','ACCUMULATED VOLUME RESIDUALS'],'FontSize',8);
xlabel('TIME [HOURS]','FontSize',8);
ylabel('(VOL_t_r_u_e - VOL_e_s_t) [INCHES]','FontSize',8);
axis([x_range,-inf,inf]); % make both plot have same axes limits.
box on;

%--------------------------------------------------------------------------
% plot discharges and residuals...
h_Q_ax = subplot(4,4,[11 12]); hold on; % discharges.
plot(Tmod,Hin.Q_OBS,'Color',[0 0.5 0]); % measured.
plot(Tmod,Hin.Q_SIM,'Color',[1 0.0 0]); % result from unit hydrograph.
title(['\bf','OBS AND ESTIMATED DISCHARGES'],'FontSize',8);
xlabel('TIME [HOURS]','FontSize',8);
ylabel('DISCHARGE [CFS]','FontSize',8);
% legend('actual discharge','estimated discharge','Location','northeast');
axis([x_range,-inf,inf]);
box on;

subplot(4,4,[15 16]); % residuals.
bar(Tmod,(Hin.Q_OBS-Hin.Q_SIM),'EdgeColor',[1 0 0], 'FaceColor',[1 0 0]); hold on;
title(['\bf','DISCHARGE RESIDUALS'],'FontSize',8);
xlabel('TIME [HOURS]','FontSize',8);
ylabel('(Q_t_r_u_e - Q_e_s_t) [CFS]','FontSize',8);
axis([x_range,-inf,inf]); % make both plot have same axes limits.
box on;

% save figure as a PDF...
set(gcf,'PaperOrientation','landscape','InvertHardcopy','on','PaperPositionMode','manual',...
        'PaperUnits','normalized','PaperPosition',[0,0,1,1],'PaperType','A4');
saveas(gcf,[Hin.outdir,pdf_name,'.pdf'],'pdf');
if ~Hin.db_obj, % close plot if not debugging, otherwise keep on screen.
    close(gcf);
end
pause(0.5);

return;

