function [Pxs,loss] = precip_loss(Ptot,Lfun,c,no_neg_P,no_neg_loss)
% USAGE: [Pxs,loss] = precip_loss(Ptot,Lfun,c)
%
% DESCRIPTION: This function applies a loss function to a total rainfall
%   time-series to get the effective rainfall.
%
% INPUT:
%   Ptot - column vector or total rainfall time-series [inches. 5min increments].
%   Lfun - string indicating which loss function to use:
%            'L1':  1-parameter linear,            L(t) = c1
%            'L2':  2-parameter linear,            L(t) = c1*t + c2
%            'L3':  3-parameter quadratic,         L(t) = c1*t^2 + c2*t + c3
%            'E1':  1-parameter exponential decay, L(t) = exp(-c1*t)
%            'E2':  2-parameter exponential decay, L(t) = c1*exp(-c2*t)
%            'E3':  3-parameter exponential decay, L(t) = c1*exp( -(c2*t + c3))
%            'E4':  4-parameter exponential decay, L(t) = c1*exp( -(c2*t + c3)) + c4
%            'P1':  1-parameter proportional loss, P(t) = P(t) - c1*P(t)
%            'A1':  1-parameter init. abstraction, P(t) = P(t) - init.abs(c1*Ptot)
%            'AP2': 2-parameter, init.abstraction then proportional loss.
%            'AC2': 2-parameter, init.abstraction then constant loss.
%   c    - scalar or vector specifying the loss function parameters, as defined above.
%   'no_neg_P' - logical flag specifying whether to allow negative
%                precipitation values after applying the loss function.
%                  1 = do not allow negative precipitation values; any negative
%                      values after applying the loss are set to 0.
%                  0 = allow negative precipitation values; do not set negative
%                      values to 0.
%   'no_neg_loss' - logical flag specifying whether to allow negative loss
%                   function values, resulting in precipitation gains.
%                     1 = do not allow negative loss function values; any negative
%                         values are set to 0 prior to its application.
%                     0 = allow negative loss function values; do not set negative
%                         values to 0.
%
% OUTPUT:
%   Pxs - rain time-series after loss function application with negative values set to 0 [inches. 5min increments].
%   loss - (optional) loss function applied [inches. 5min increments].
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: [Pxs,Punfilt] = precip_loss(Ptot,Lfun,c)';

% check arguments...
if nargin ~= 5,
    disp(usage);
    error('Incorrect number of input arguments.');
end

% get time vector...
T = (5/60)*(0:length(Ptot)-1)';

% define loss function...
switch upper(Lfun)
    case 'L1', % L(t) = c1
        loss = c(1).*ones(size(Ptot));
    case 'L2', % L(t) = c1*t + c2
        loss = c(1).*T + c(2);
    case 'L3', % L(t) = c1*t^2 + c2*t + c3
        loss = c(1).*(T.^2) + c(2).*T + c(3);
    case 'E1', % L(t) = exp(-c1*t)
        loss = exp( -c(1).*T );
    case 'E2', % L(t) = c1*exp(-c2*t)
        loss = c(1) .* exp( -c(2).*T ); % original
%         loss = exp( -c(1).*T ) + c(2);
    case 'E3', % L(t) = c1*exp( -(c2*t + c3))
        loss = c(1) .* exp( -( c(2).*T + c(3) ) );
    case 'E4', % L(t) = c1*exp( -(c2*t + c3)) + c4
        loss = c(1) .* exp( -( c(2).*T + c(3) ) ) + c(4);
    case 'P1', % Pxs(t) = c(1)*Ptot(t), 0 <= c(1) <= 1 (1-parm proportional loss)
        loss = Ptot - c(1).*Ptot;
    case 'A1', % P(t) = P(t) - init.abs(c1)
        % c1 is frac of total precip volume to initially abstract (should be between 0 and 1)
        loss = init_abs(Ptot,c); % separate function below, independent of time.
    case 'AC2', % init.abs. then const.loss.
        % apply init.abs to get intermediate rain...
        Pint = Ptot - init_abs(Ptot,c(1)); % 1st c does init.abs.
        % apply const.loss to this...
        Pint = Pint - c(2).*ones(size(Ptot)); % 2nd c does const.loss.
        % get overall loss that occurred...
        loss = Ptot - Pint;
    case 'AP2', % init.abs. then prop.loss.
        % apply init.abs to get intermediate rain...
        Pint = Ptot - init_abs(Ptot,c(1)); % 1st c does init.abs.
        % apply prop.loss to this...
        Pint = c(2).*Pint; % 2nd c does prop.loss.
        % get overall loss that occurred...
        loss = Ptot - Pint;
    otherwise  % error.
        error(['Unrecognized loss function specified: ',Lfun,'.']);
end

if no_neg_loss, % do not allow negative loss - would increase precip...
    loss(loss<0) = 0;
end

% apply loss to get excess...
Pxs = Ptot - loss;

if no_neg_P, % set any final negative values to 0 - can't have neg precip...
    Pxs(Pxs<0) = 0;
end

% get final applied loss after adjustments...
loss = Ptot-Pxs;

% % % for testing...
% % plot(1:length(L),L,'r')
% % bar(Ptot); hold on;
% % bar(Pxs);
% % xlabel('TIME: 5min increments');
% % ylabel('PRECIP [INCHES]');
% % legend('loss function','total precip','excess precip');
% % DEBUG...
% figure; hold on;
% bar(T,Ptot,    'EdgeColor',[0.2 0.5 0.2], 'FaceColor',[0.2 0.5 0.2]); % total modeled.
% bar(T,Pxs,     'EdgeColor',[1.0 0.0 0.0], 'FaceColor',[1.0 0.0 0.0]); % excess.
% plot(T,loss,'b');
% plot(T,Ptot-Pxs,'r');
% error('???')


return;

%==========================================================================
% AUXILARY FUNCTIONS
function Ploss = init_abs(Ptot,c)
% USAGE: Ploss = init_abs(Ptot,c)
%
% DESCRIPTION: This function initially abstracts the fraction c of the total
%   input precip volume from the input precip time series and returns the
%   LOSS FUNCTION time-series (Ploss = Ptot-Pabs).
%
% INPUT:
%   Ptot - numeric column vector or total rainfall time-series [inches. 5min increments].
%   c    - numeric fraction of the total sum of Ptot to initailly abstract.
%
% OUTPUT:
%   Pabs - numeric column vector of resulting initial abstraction.
%
% Written by: Joe Vrabel, USGS July2006

remove = c*sum(Ptot); % amount to initially abstract.

% special cases...
if c >= 1, % remove all precip.
    Pabs = zeros(size(Ptot));
    Ploss = Ptot - Pabs; % return loss time-series.
    return; % done.
end
if c < 0, % indicates ADD precip - just return original Ptot.
    Pabs = Ptot;
    Ploss = Ptot - Pabs; % return loss time-series.
    return;
end
if Ptot(1) > remove, % all abstraction from 1st value.
    Pabs = Ptot;
    Pabs(1) = Pabs(1) - remove;
    Ploss = Ptot - Pabs; % return loss time-series.
    return;
end

% % normal cases... slower.
% Pabs = Ptot;
% partial_sum = 0;
% for i = 1:length(Ptot),
%     partial_sum = partial_sum + Ptot(i); % incremental partial sums.
%     if partial_sum > remove, % found index where partial sum is greater than what needs to be removed.
%         Pabs(1:i-1) = 0; % set precip up to preceeding index to 0.
%         remainder = remove - sum(Ptot(1:i-1)); % find remaining precip to remove.
%         Pabs(i) = Pabs(i) - remainder; % remove remainder.
%         Ploss = Ptot - Pabs; % return loss time-series.
%         return; % done.
%     end
% end

% normal cases... fastest method - need check.
Pabs = Ptot;
index = sum(cumsum(Ptot)<=remove); % rain up to here needs to be removed.
Pabs(1:index) = 0; % abstract it.
remainder = remove - sum(Ptot(1:index)); % find any remaining precip to remove.
Pabs(index+1) = Pabs(index+1) - remainder; % remove remainder.
Ploss = Ptot - Pabs; % return loss time-series.

return;

