function [Hin,err] = run_model(Hin,c)
% USAGE: [Hin,err] = run_model(Hin,c)
%
% DESCRIPTION: This function runs a unit hydrograph model.
%
% INPUT:
%   Hin - data structure containing all needed data.
%   c   - precip loss model parameters to use.
%
% OUTPUT:
%   Hin - updated data structure including model results.
%
% Written by: Joe Vrabel, USGS June2006

err = 0;

[Hin.RAIN_XS,Hin.loss] = precip_loss(Hin.RAIN_USE,Hin.Lfun,c,Hin.no_neg_P,Hin.no_neg_loss); % apply loss function to get excess rain.

% generate unit hydrograph...
[Hin.K]    = calc_K(Hin.LENGTH,Hin.DEVEL); % find gamma function shape parameter [dimensionless].
[Hin.Tp]   = calc_Tp(Hin.SLOPE,Hin.LENGTH,Hin.DEVEL); % find time to peak [hrs].
[Hin.qp]   = calc_qp(Hin.K,Hin.Tp,Hin.AREA); % find peak discharge [CFS].
% Tunit  = (0:(5/60):(10*Hin.RAIN_DUR))'; % use time vector that is 10 times the duration of the storm event [hrs in 5min increments]
Tunit = get_Tunit(Hin.qp,Hin.Tp,Hin.K,Hin.fracQp); % use time vector that lasts until fracQp*qp has been reached [hrs in 5min increments].
[U]    = calc_U(Tunit,Hin.qp,Hin.Tp,Hin.K); % generate unit hydrograph [CFS].

% check array size for memory problems before creating convolution matrix...
array_size_MB = 8*length(U)*(length(U)+length(Hin.RAIN_XS)-1)/1e6;
if array_size_MB > Hin.max_array_size_MB, % exceeds max in MB.
    disp(['MAXIMUM ALLOWABLE ARRAY SIZE (',num2str(Hin.max_array_size_MB),'MB) EXCEEDED: ',num2str(array_size_MB),'MB']);
    err = 1;
    return;
end

[Hin.Q_SIM,A]    = calc_Q(U,Hin.RAIN_XS); % calculate discharge using unit hydrograph and excess rain found [CFS].
[Hin.V_SIM,Vtot] = calc_vol(Hin.Q_SIM,Hin.AREA); % calculate simulated volume [inches].

% pad time-series so sim/obs match up in time...
if Hin.pad <= 0, % pad actual with leading zeros.
    Hin.V_OBS    = [zeros(abs(Hin.pad),1);Hin.V_OBS];
    Hin.Q_OBS    = [zeros(abs(Hin.pad),1);Hin.Q_OBS];
%     Hin.RAIN_USE = [zeros(abs(Hin.pad),1);Hin.RAIN_USE];
%     Hin.RAIN_XS  = [zeros(abs(Hin.pad),1);Hin.RAIN_XS];
else % pad model with leading zeros.
    Hin.V_SIM = [zeros(Hin.pad,1);Hin.V_SIM];
    Hin.Q_SIM = [zeros(Hin.pad,1);Hin.Q_SIM];
end

% trim/pad sim/obs time-series...
switch lower(Hin.sim_obs_trim_pad_meth)
    case 'trim_pad_to_obs',
        % post-pad or trim SIM to same size as OBS:
        %   Q gets post-padded with 0's.
        %   V gets post-padded with it's last value (ie: doesn't go to 0 but plateaus).
        if     length(Hin.V_SIM) < length(Hin.V_OBS), % OBS bigger.
            Hin.Q_SIM = [Hin.Q_SIM;              zeros(length(Hin.Q_OBS)-length(Hin.Q_SIM),1)]; % pad SIM Q.
            Hin.V_SIM = [Hin.V_SIM;Hin.V_SIM(end)*ones(length(Hin.V_OBS)-length(Hin.V_SIM),1)]; % pad SIM V.
        elseif length(Hin.V_SIM) > length(Hin.V_OBS), % SIM bigger.
            Hin.Q_SIM = Hin.Q_SIM(1:length(Hin.Q_OBS)); % trim SIM Q.
            Hin.V_SIM = Hin.V_SIM(1:length(Hin.V_OBS)); % trim SIM V.
        end
    case 'trim_pad_to_sim',
        % post-pad or trim OBS to same size as SIM:
        %   Q gets post-padded with 0's.
        %   V gets post-padded with it's last value (ie: doesn't go to 0 but plateaus).
        if     length(Hin.V_OBS) < length(Hin.V_SIM), % SIM bigger.
            Hin.Q_OBS = [Hin.Q_OBS;              zeros(length(Hin.Q_SIM)-length(Hin.Q_OBS),1)]; % pad OBS Q.
            Hin.V_OBS = [Hin.V_OBS;Hin.V_OBS(end)*ones(length(Hin.V_SIM)-length(Hin.V_OBS),1)]; % pad OBS V.
        elseif length(Hin.V_OBS) > length(Hin.V_SIM), % OBS bigger.
            Hin.Q_OBS = Hin.Q_OBS(1:length(Hin.Q_SIM)); % trim OBS Q.
            Hin.V_OBS = Hin.V_OBS(1:length(Hin.V_SIM)); % trim OBS V.
        end
    case 'trim_to_shorter',
        % trims larger time-series to smaller.
        if length(Hin.V_SIM) < length(Hin.V_OBS), % obs bigger.
            Hin.Q_OBS = Hin.Q_OBS(1:length(Hin.Q_SIM)); % trim obs Q to sim.
            Hin.V_OBS = Hin.V_OBS(1:length(Hin.V_SIM)); % trim obs V to sim.
        else                                      % sim bigger.
            Hin.Q_SIM = Hin.Q_SIM(1:length(Hin.Q_OBS)); % trim sim Q to obs.
            Hin.V_SIM = Hin.V_SIM(1:length(Hin.V_OBS)); % trim sim Q to obs.
        end
    case 'pad_to_longer',
        % pads smaller to larger.
        if length(Hin.V_SIM) < length(Hin.V_OBS), % obs bigger.
            Hin.Q_SIM = [Hin.Q_SIM;              zeros(length(Hin.Q_OBS)-length(Hin.Q_SIM),1)]; % pad SIM Q.
            Hin.V_SIM = [Hin.V_SIM;Hin.V_SIM(end)*ones(length(Hin.V_OBS)-length(Hin.V_SIM),1)]; % pad SIM V.
        else                                      % sim bigger.
            Hin.Q_OBS = [Hin.Q_OBS;              zeros(length(Hin.Q_SIM)-length(Hin.Q_OBS),1)]; % pad OBS Q.
            Hin.V_OBS = [Hin.V_OBS;Hin.V_OBS(end)*ones(length(Hin.V_SIM)-length(Hin.V_OBS),1)]; % pad OBS V.
        end
    otherwise
        disp(['Unrecognized ''sim_obs_trim_pad_meth'' value: ',Hin.sim_obs_trim_pad_meth]);
        err = 1;
        return;
end

return;


