function [P] = trim_precip(P,P_trim1,P_trim2)
% USAGE: [P] = trim_precip(P,P_trim1,[P_trim2])
%
% DESCRIPTION: This function takes an interpolated accumulated precip
%   time-series, finds the total volume of precip, and trims the fractions
%   of volume from the beginning and end of the event as specified by
%   P_trim1 and P_trim2, respectively.
%
% INPUT:
%   P - (numeric vector) precip time series, given by: [time,value].
%   P_trim1 - (numeric scalar) fraction between 0 and 1 of total volume to
%             trim from the beginning of the precip event.
%   P_trim2 - (numeric scalar, optional) fraction between 0 and 1 of total
%             volume to trim from the end of the precip event. If omitted,
%             P_trim2 is set to P_trim1 for symetrical trimming.
%
% OUTPUT:
%   P - (numerical vector) the trimmed precip time-series.
%
% Written by: Joe Vrabel, USGS June2006

usage = 'USAGE: [P] = trim_precip(P,P_trim1,[P_trim2])';

% check input args...
if nargin < 3,
    disp(usage);
    error('Not enough input arguments.');
end
if nargin == 2,
    P_trim2 = P_trim1; % symetrical trimming.
end
if nargin > 3,
    disp(usage);
    error('Too many input arguments.');
end
if (~isnumeric(P)) || (size(P,2) ~= 2),
    disp(usage);
    error('Input argument ''P'' must be a numeric array with 2 columns.');
end
if (~isnumeric(P_trim1)) || (numel(P_trim1) ~= 1),
    disp(usage);
    error('Input argument ''P_trim1'' must be a numeric scalar between 0 and 1 inclusive.');
end
if (P_trim1 < 0) || (P_trim1 > 1),
    disp(usage);
    error('Input argument ''P_trim1'' must be a numeric scalar between 0 and 1 inclusive.');
end
if (~isnumeric(P_trim2)) || (numel(P_trim2) ~= 1),
    disp(usage);
    error('Input argument ''P_trim2'' must be a numeric scalar.');
end
if (P_trim2 < 0) || (P_trim2 > 1),
    disp(usage);
    error('Input argument ''P_trim2'' must be a numeric scalar between 0 and 1 inclusive.');
end

% get total change of precip volume...
deltaP = P(end,2) - P(1,2);

% find trim index of leading tail...
for i = 1:size(P,1),
    if (P(i,2) - P(1,2)) > P_trim1*deltaP,
        trim_index1 = i;
        break;
    end
end

% find trim index of trailing tail...
for i = size(P,1):-1:1,
    if (P(end,2) - P(i,2)) > P_trim1*deltaP,
        trim_index2 = i;
        break;
    end
end

% Porig = P;
% trim trailing tail...
P(trim_index2:end,:) = [];

% replace leading tail values with zeros...
% P(1:trim_index1-1,2) = 0;
P(1:trim_index1-1,:) = [];

% plot(Porig);  hold on; plot(P,'r.');error('wer')
return;

