function [Hin] = write_out(Hin,state)
% USAGE: [Hin] = write_out(Hin,[''state''])
%
% DESCRIPTION: This function writes model results to the output file.
%   For each run, the output file is opened and a line of output data
%   for the run is appended. Note that the original file before the first
%   run should already have the text header line.
%
% INPUT:
%   Hin   - data structure containing all neccessary output data.
%   state - (optional, string):
%             ** if set to 'null', RUN_NUM, DATADIR, STAID, YYYYMMDD, AREA{mi^2],
%                CHAN_LENGTH[mi], SLOPE, snd DEVELOPED will be written to
%                the output file for the current run, with null results for
%                all other output values. Use when a run is skipped in the batch.
%             ** if set to 'values' or not specified, the non-null run
%                results values will be output.
%
% OUTPUT:
%   An appended line to the ouput file.
%
%   Also, some output calculated in this function is added to the Hin data
%   structure for use in later plotting.
%
% Written by: Joe Vrabel, USGS July2006

usage = 'USAGE: [Hin] = write_out(Hin,[''state''])';

% check input...
if (nargin ~= 1) && (nargin ~= 2)
    disp(usage);
    error('Incorrect number of input arguments.');
end
if nargin == 1, % default verbose.
    state = 'values'; % not null.
end
if ~isstruct(Hin),
    disp(usage);
    error('Input ''Hin'' must be a data structure.');
end
if ~strcmp(state,'null') && ~strcmp(state,'values'),
    disp(usage);
    error('When specified, input ''state'' must be set to ''null'' or ''values''.');
end

%--------------------------------------------------------------------------
% list of output variables (for text header):
% RUN_NUM
% DATADIR
% STAID
% YYYYMMDD
% AREA[mi^2]
% CHAN_LENGTH[mi]
% SLOPE
% DEVELOPED
% GAMMA_Tp[hr]
% GAMMA_Qp[CFS]
% GAMMA_Qp[in/hr]
% GAMMA_K
% N_MOD_PEAKS
% N_POINTS
% BEGIN_RAIN
% RAIN_TOT_VOL[in]
% RAIN_XS_VOL[in]
% RAIN_PER_VOL_LOSS
% TpOBS_TpSIM_DIFF[hrs]
% OBS_Q_MEAN[CFS]
% SIM_Q_MEAN[CFS]
% OBS_Q_MAX[CFS]
% SIM_Q_MAX[CFS]
% RMS_Q_RESID[CFS]
% Q_ERR_VAR[CFS^2]
% Q_REL_BIAS
% Q_REL_STD_ERR
% Q_NS_EFF
% Q_SIM_OBS_R2
% Q_SIM_OBS_SLOPE
% Q_SIM_OBS_INT
% OBS_V_MEAN[CFS]
% SIM_V_MEAN[CFS]
% OBS_V_MAX[CFS]
% SIM_V_MAX[CFS]
% OBS_V_TOT[in]
% SIM_V_TOT[in]
% RMS_V_RESID[CFS]
% V_ERR_VAR[CFS^2]
% V_REL_BIAS
% V_REL_STD_ERR
% V_NS_EFF
% V_SIM_OBS_R2
% V_SIM_OBS_SLOPE
% V_SIM_OBS_INT
% PER_VOL_MATCH
% LOSS_MODEL
% OBJ_FUNC
% OBJ_OPT
% C_OPT
%--------------------------------------------------------------------------

% open output file in 'append' mode ...
fid = fopen(Hin.outfile,'a');

                % MISC INFO
% add RUN_NUM...
fprintf(fid,'%s',num2str(Hin.run_num));
fprintf(fid,'%s',',');
                
% add DATADIR...
slashes = findstr(Hin.hydro_file,'\');
fname = Hin.hydro_file(slashes(end)+1:end);
Hin.DATADIR = Hin.hydro_file(1:slashes(end-2)); % note there is also a Hin.datadir.
fprintf(fid,'%s',Hin.DATADIR);
fprintf(fid,'%s',',');

% add STAID...
fprintf(fid,'%s',fname(9:16));
fprintf(fid,'%s',',');

% add YYYYMMDD...
fprintf(fid,'%s',[fname(18:21),fname(23:26)]);
fprintf(fid,'%s',',');

% add AREA[mi^2]...
fprintf(fid,'%s',num2str(Hin.AREA));
fprintf(fid,'%s',',');

% add CHAN_LENGTH[mi]...
fprintf(fid,'%s',num2str(Hin.LENGTH));
fprintf(fid,'%s',',');

% add SLOPE...
fprintf(fid,'%s',num2str(Hin.SLOPE));
fprintf(fid,'%s',',');

% add DEVELOPED...
fprintf(fid,'%s',num2str(Hin.DEVEL));
fprintf(fid,'%s',',');

if strcmp(state,'null'), % output null for rest...
    fprintf(fid,'%s',char(44*ones(1,49))); % nulls for all but c_opt (49 commas).
    for i = 1:numel(Hin.c0)-1, % may be variable number of c_opt's.
        fprintf(fid,'%s',',');
    end
    fprintf(fid,'%s\n',''); % add end-of-line character.
    fclose(fid);
    return;
end

                % UNIT HYDROGRAPH PARMS
% add GAMMA_Tp[hr]...
fprintf(fid,'%s',num2str(Hin.Tp));
fprintf(fid,'%s',',');

% add GAMMA_Qp[CFS]...
fprintf(fid,'%s',num2str(Hin.qp));
fprintf(fid,'%s',',');

% add GAMMA_Qp[in/hr]...
fprintf(fid,'%s',num2str(cfs2inhr(Hin.qp,Hin.AREA)));
fprintf(fid,'%s',',');

% add GAMMA_K...
fprintf(fid,'%s',num2str(Hin.K));
fprintf(fid,'%s',',');

% add N_MOD_PEAKS...
N_MOD_PEAKS = find_n_peaks(Hin.Q_SIM,'pos',1e-3); % positive peaks only, peaks smaller than 0.1% total range ignored.
fprintf(fid,'%s',num2str(N_MOD_PEAKS));
fprintf(fid,'%s',',');

% add N_POINTS...
Hin.N = length(Hin.Q_OBS); % number of data points in sim/obs Q and vol.
fprintf(fid,'%s',num2str(Hin.N));
fprintf(fid,'%s',',');

                % RAIN STATS
% add BEGIN_RAIN...
fprintf(fid,'%s',datestr(Hin.event_start,31)); % YYYY-MM-DD HH:MM:SS
fprintf(fid,'%s',',');

% add RAIN_TOT_VOL[in]...
Hin.RAIN_TOT_VOL = sum(Hin.RAIN_USE);
fprintf(fid,'%s',num2str(Hin.RAIN_TOT_VOL));
fprintf(fid,'%s',',');

% add RAIN_XS_VOL[in]...
Hin.RAIN_XS_VOL = sum(Hin.RAIN_XS);
fprintf(fid,'%s',num2str(Hin.RAIN_XS_VOL));
fprintf(fid,'%s',',');

% add RAIN_PER_VOL_LOSS...
Hin.RAIN_PER_VOL_LOSS = 100*((Hin.RAIN_TOT_VOL - Hin.RAIN_XS_VOL)/Hin.RAIN_TOT_VOL);
fprintf(fid,'%s',num2str(Hin.RAIN_PER_VOL_LOSS));
fprintf(fid,'%s',',');

                % DISCHARGE STATS

% add TpOBS_TpSIM_DIFF[hrs]...
[junk,iQpOBS] = max(Hin.Q_OBS); % find index where obs max Q occurs.
[junk,iQpSIM] = max(Hin.Q_SIM); % find index where sim max Q occurs.
fprintf(fid,'%s',num2str((iQpOBS-iQpSIM)/12)); % convert index difference of maxes to hours.
fprintf(fid,'%s',',');

% add OBS_Q_MEAN[CFS]...
Hin.OBS_Q_MEAN = mean(Hin.Q_OBS);
fprintf(fid,'%s',num2str(Hin.OBS_Q_MEAN));
fprintf(fid,'%s',',');

% add SIM_Q_MEAN[CFS]...
Hin.SIM_Q_MEAN = mean(Hin.Q_SIM);
fprintf(fid,'%s',num2str(Hin.SIM_Q_MEAN));
fprintf(fid,'%s',',');

% add OBS_Q_MAX[CFS]...
fprintf(fid,'%s',num2str(max(Hin.Q_OBS)));
fprintf(fid,'%s',',');

% add SIM_Q_MAX[CFS]...
fprintf(fid,'%s',num2str(max(Hin.Q_SIM)));
fprintf(fid,'%s',',');

% add RMS_Q_RESID[CFS]...
Hin.RMS_Q_RESID = sqrt( mean( (Hin.Q_SIM-Hin.Q_OBS).^2 ) );
fprintf(fid,'%s',num2str(Hin.RMS_Q_RESID));
fprintf(fid,'%s',',');

% add Q_ERR_VAR[CFS^2]...
Q_ERR_VAR = sum( (Hin.Q_SIM-Hin.Q_OBS).^2 ) / (Hin.N-1);
fprintf(fid,'%s',num2str(Q_ERR_VAR));
fprintf(fid,'%s',',');

% add Q_REL_BIAS...
Hin.Q_REL_BIAS = ( sum(Hin.Q_SIM-Hin.Q_OBS) / Hin.N ) / mean(Hin.Q_OBS);
fprintf(fid,'%s',num2str(Hin.Q_REL_BIAS));
fprintf(fid,'%s',',');

% add Q_REL_STD_ERR...
Q_REL_STD_ERR = Q_ERR_VAR / ( sum( (Hin.Q_OBS - mean(Hin.Q_OBS) ).^2 ) / (Hin.N-1) );
fprintf(fid,'%s',num2str(Q_REL_STD_ERR));
fprintf(fid,'%s',',');

% add Q_NS_EFF...
Hin.Q_NS_EFF = 1-Q_REL_STD_ERR;
fprintf(fid,'%s',num2str(Hin.Q_NS_EFF));
fprintf(fid,'%s',',');

% add Q_SIM_OBS_R2...
[Hin.Q_SIM_OBS_SLOPE,Hin.Q_SIM_OBS_INT,Hin.Q_SIM_OBS_R2] = regress_mbr(Hin.Q_SIM,Hin.Q_OBS); % SIM vs OBS.
fprintf(fid,'%s',num2str(Hin.Q_SIM_OBS_R2));
fprintf(fid,'%s',',');

% add Q_SIM_OBS_SLOPE...
fprintf(fid,'%s',num2str(Hin.Q_SIM_OBS_SLOPE));
fprintf(fid,'%s',',');

% add Q_SIM_OBS_INT...
fprintf(fid,'%s',num2str(Hin.Q_SIM_OBS_INT));
fprintf(fid,'%s',',');

                % VOLUME STATS
                
% add OBS_V_MEAN[CFS]...
Hin.OBS_V_MEAN = mean(Hin.V_OBS);
fprintf(fid,'%s',num2str(Hin.OBS_V_MEAN));
fprintf(fid,'%s',',');

% add SIM_V_MEAN[CFS]...
Hin.SIM_V_MEAN = mean(Hin.V_SIM);
fprintf(fid,'%s',num2str(Hin.SIM_V_MEAN));
fprintf(fid,'%s',',');

% add OBS_V_MAX[CFS]...
fprintf(fid,'%s',num2str(max(Hin.V_OBS)));
fprintf(fid,'%s',',');

% add SIM_V_MAX[CFS]...
fprintf(fid,'%s',num2str(max(Hin.V_SIM)));
fprintf(fid,'%s',',');

% add OBS_V_TOT[in]...
fprintf(fid,'%s',num2str(Hin.V_OBS(end)));
fprintf(fid,'%s',',');

% add SIM_V_TOT[in]...
fprintf(fid,'%s',num2str(Hin.V_SIM(end)));
fprintf(fid,'%s',',');

% add RMS_V_RESID[CFS]...
Hin.RMS_V_RESID = sqrt( mean( (Hin.V_SIM-Hin.V_OBS).^2 ) );
fprintf(fid,'%s',num2str(Hin.RMS_V_RESID));
fprintf(fid,'%s',',');

% add V_ERR_VAR[CFS^2]...
V_ERR_VAR = sum( (Hin.V_SIM-Hin.V_OBS).^2 ) / (Hin.N-1);
fprintf(fid,'%s',num2str(V_ERR_VAR));
fprintf(fid,'%s',',');

% add V_REL_BIAS...
Hin.V_REL_BIAS = ( sum(Hin.V_SIM-Hin.V_OBS) / Hin.N ) / mean(Hin.V_OBS);
fprintf(fid,'%s',num2str(Hin.V_REL_BIAS));
fprintf(fid,'%s',',');

% add V_REL_STD_ERR...
V_REL_STD_ERR = V_ERR_VAR / ( sum( (Hin.V_OBS - mean(Hin.V_OBS) ).^2 ) / (Hin.N-1) );
fprintf(fid,'%s',num2str(V_REL_STD_ERR));
fprintf(fid,'%s',',');

% add V_NS_EFF...
Hin.V_NS_EFF = 1-V_REL_STD_ERR;
fprintf(fid,'%s',num2str(Hin.V_NS_EFF));
fprintf(fid,'%s',',');

% add V_SIM_OBS_R2...
[Hin.V_SIM_OBS_SLOPE,Hin.V_SIM_OBS_INT,Hin.V_SIM_OBS_R2] = regress_mbr(Hin.V_SIM,Hin.V_OBS); % SIM vs OBS.
fprintf(fid,'%s',num2str(Hin.V_SIM_OBS_R2));
fprintf(fid,'%s',',');

% add V_SIM_OBS_SLOPE...
fprintf(fid,'%s',num2str(Hin.V_SIM_OBS_SLOPE));
fprintf(fid,'%s',',');

% add V_SIM_OBS_INT...
fprintf(fid,'%s',num2str(Hin.V_SIM_OBS_INT));
fprintf(fid,'%s',',');

                % OPTIMIZATION STUFF

% add SIM_OBS_TOT_VOL_RATIO...
Hin.SIM_OBS_TOT_VOL_RATIO = Hin.V_SIM(end)/Hin.V_OBS(end);
fprintf(fid,'%s',num2str(Hin.SIM_OBS_TOT_VOL_RATIO));
fprintf(fid,'%s',',');

% add LOSS_MODEL...
out_Lstr = Hin.Lstr;
out_Lstr(findstr(out_Lstr,'_')) = []; % get rid of '_' for output.
fprintf(fid,'%s',out_Lstr);
fprintf(fid,'%s',',');

% add OBJ_FUNC...
out_obj_str = Hin.obj_str;
out_obj_str(findstr(out_obj_str,'_')) = []; % get rid of '_' for output.
out_obj_str(findstr(out_obj_str,'\')) = []; % get rid of '\' for output.
fprintf(fid,'%s',out_obj_str);
fprintf(fid,'%s',',');

% add OBJ_OPT...
fprintf(fid,'%s',num2str(Hin.obj_opt));
fprintf(fid,'%s',',');

% add C_OPT_OUT...
for i = 1:numel(Hin.c_opt_out)-1, % may be variable number of c_opt's.
    fprintf(fid,'%s',num2str(Hin.c_opt_out(i)));
    fprintf(fid,'%s',',');
end
fprintf(fid,'%s',num2str(Hin.c_opt_out(numel(Hin.c_opt_out)))); % add last one.

%--------------------------------------------------------------------------
% add end-of-line character...
fprintf(fid,'%s\n','');

% close file...
fclose(fid);

return;

