function echo_run_info(user_args,Hin)
% USAGE: echo_run_info(user_args,Hin)
%
% DESCRIPTION: This function echos all run parameters for a batch_unit run
%   to the screen.
%
% INPUT:
%   user_args - cell array containing user-specified command-line input arguments to batch_unit (from varargin).
%   Hin - data structure containing all run parameters.
%
% OUTPUT:
%   Screen echo of run parameters with details.
%
% Written by: Joe Vrabel, USGS July2006

disp('==============================================================');
disp('BATCH_UNIT: UNIT HYDROGRAPH PRECIPITATION LOSS FUNCTION OPTIMIZATION');
disp(' ');
disp(['RUN STARTED AT: ',datestr(now)]);
disp(' ');

% echo explicitly specified user input parms...
disp('USER SPECIFIED COMMAND-LINE PARAMETERS:');
if ~isempty(user_args),
    disp('   PropertyName                  property_value');
    disp('   ------------                  --------------');
    for i = 1:2:length(user_args),
        % pad values for pretty display...
        PropertyName = blanks(30); PropertyName(1:length(user_args{i})+2) = ['''',user_args{i},''''];
        property_value = num2str(user_args{i+1});
        disp(['   ',PropertyName,property_value])
    end
else
    disp('   NONE');
end
disp(' ');

% echo raw data structure...
disp('INPUT DATA STRUCTURE Hin:');
disp(Hin);
disp(' ');

% give details...
disp('INPUT DETAILS:');
disp(' ');

disp('        ** FILE AND DIRECTORY CONTROL **');
disp(['* Input data file directory: ',Hin.datadir]);
disp(['* Output directory: ',Hin.outdir,' (if this directoy already exists, its contents will be deleted)']);
disp(['* Output file: ',Hin.outfile]);
disp(['* Log of batch will be written to: ',Hin.logfile]);
disp(['* Watershed characteristics loaded from: ',Hin.charfile]);
disp(' ');

disp('        ** PRECIPITATION PARAMETERS **');
if Hin.use_synthP,
    disp('* Synthetic rainfall used.');
else
    disp('* Actual rainfall used.');
end
disp(['* Precip leading tail trim level: ',num2str(100*Hin.P_trim1),'%']);
disp(['* Precip trailing tail trim level: ',num2str(100*Hin.P_trim2),'%']);
if Hin.no_neg_P,
    disp('* No negative precip allowed - all P<0 set to 0 after loss function application.');
else
    disp('* Negative precip allowed.');
end
if Hin.no_neg_loss,
    disp('* No negative loss function values allowed - all loss<0 set to 0 prior to application.');
else
    disp('* Negative loss function values allowed.');
end
disp(' ');

disp('        ** OPTIMIZATION PARAMETERS **');
disp(['* "',Hin.Lfun,'" precip loss function used.']);
switch lower(Hin.opt_meth)
    case 'fmin'
        disp(['* Objective function ',num2str(Hin.obj),' used.']);
        disp( '* Optimization using ''fminsearch'' specified.');
        disp( '    * Initial precip loss function parameter(s) used:');
        for i = 1:length(Hin.c0),
            disp(['      c(',num2str(i),') = ',num2str(Hin.c0(i))]);
        end
        disp( '    * Parameter search space is unrestricted.');
        disp(['    * Maximum randomization iterations allowed: ',num2str(Hin.max_monte)]);
        disp(['    * Randomization lower bound: ',num2str(Hin.min_c)]);
        disp(['    * Randomization upper bound: ',num2str(Hin.max_c)]);
        disp(['    * Maximum allowable optimization iterations: ',num2str(Hin.fmin_max_its)]);
        disp(['    * Objective function optimization tolerance: ',num2str(Hin.fmin_obj_tol)]);
        disp( 'NOTE: Parameters ''n_c1'', ''n_c2'', and ''grid_tols'' will have no effect in this batch run.');
    case 'fbnd'
        disp(['* Objective function ',num2str(Hin.obj),' used.']);
        disp( '* Optimization using ''fminsearch'' specified.');
        disp( '    * Initial precip loss function parameter(s) used:');
        for i = 1:length(Hin.c0),
            disp(['      c(',num2str(i),') = ',num2str(Hin.c0(i))]);
        end
        disp( '    * Parameter search space restricted to:');
        disp(['      lower bounds: ',num2str(Hin.min_c)]);
        disp(['      upper bounds: ',num2str(Hin.max_c)]);
        disp(['    * Maximum randomization iterations allowed: ',num2str(Hin.max_monte)]);
        disp(['    * Randomization lower bound: ',num2str(Hin.min_c)]);
        disp(['    * Randomization upper bound: ',num2str(Hin.max_c)]);
        disp(['    * Maximum allowable optimization iterations: ',num2str(Hin.fmin_max_its)]);
        disp(['    * Objective function optimization tolerance: ',num2str(Hin.fmin_obj_tol)]);
        disp( 'NOTE: Parameters ''n_c1'', ''n_c2'', and ''grid_tols'' will have no effect in this batch run.');
    case 'grid_search'
        disp(['* Objective function ',num2str(Hin.obj),' used.']);
        disp( '* Optimization using ''grid_search'' specified.');
        disp(['    * Number of grid search passes: ',num2str(length(Hin.grid_tols))]);
        disp(['    * Initial parameter space lower bound: ',num2str(Hin.min_c)]);
        disp(['    * Initial parameter space upper bound: ',num2str(Hin.max_c)]);
        disp(['    * Successive pass relative volume match tolerances: ',num2str(Hin.grid_tols)]);
        disp('NOTE: Parameters ''fmin_max_its'', ''fmin_obj_tol'', ''max_monte'', and ''c0'' will have no effect in this batch run.');
    case 'none'
        disp('* No optimization performed - model will be run once using c0 specified to get results.')
end
disp(' ');

disp('        ** MISCELLANEOUS **');
if Hin.save_pdf,
    disp('* Individual PDF images will be saved for this batch run in the output directory.');
else
    disp('* No PDF images will be saved for this batch run.');
end
disp(['* Starting batch at run #',num2str(Hin.start_stop(1)),'.']);
if isinf(Hin.start_stop(2)),
    disp('* Running batch until completion.');
else
    disp(['* Stopping batch at run #',num2str(Hin.start_stop(2)),'.']);
end
if strcmp(Hin.USGS_DEM_AREA,'USGS'),
    disp('* USGS watershed areas used.');
elseif strcmp(Hin.USGS_DEM_AREA,'DEM'),
    disp('* DEM watershed areas used.');
else
    error(['Unexpected USGS_DEM_AREA value: ',Hin.USGS_DEM_AREA]);
end
disp(['* Maximum allowable convolution matrix size: ',num2str(Hin.max_array_size_MB),' MegaBytes.']);
disp(['* Model will run until gamma unit hydrograph decays to ',num2str(100*Hin.fracQp),'% of Qp.']);
switch lower(Hin.sim_obs_trim_pad_meth)
    case 'trim_pad_to_obs'
        disp('* Simulated volume and discharge data will be padded or trimmed to the same length as the observed data.');
    case 'trim_pad_to_sim'
        disp('* Observed volume and discharge data will be padded or trimmed to the same length as the simulated data.');
    case 'pad_to_longer'
        disp('* The shorter of the simulated/observed volume and discharge data will be padded to the longer of the two.');
    case 'trim_to_shorter'
        disp('* The longer of the simulated/observed volume and discharge data will be trimmed to the shorter of the two.');
end
if Hin.db_obj,
    disp('* Debug mode ON - objective space plotting will occur terminating the batch.');
else
    disp('* Debug mode OFF - normal batch mode.');
end
disp(' ')

return;

